from dolfin import *

# Create mesh and define function space
mesh = UnitCubeMesh(32, 32, 32)

marker = MeshFunction("size_t", mesh, mesh.topology().dim()-2, 0)
for e in edges(mesh):
    marker[e] = 0.5 - DOLFIN_EPS < e.midpoint().z() < 0.5 + DOLFIN_EPS and 0.5 - DOLFIN_EPS < e.midpoint().y() < 0.5 + DOLFIN_EPS

submesh = MeshView.create(marker, 1)

V1 = FunctionSpace(mesh, "Lagrange", 1) ## 3D
V2 = FunctionSpace(submesh, "Lagrange", 1) ## 1D

# Define Dirichlet boundary (x = 0 or x = 1)
def boundary(x):
    return x[0] < DOLFIN_EPS or x[0] > 1.0 - DOLFIN_EPS

# Define boundary condition
u0 = Constant(0.0)
# 3D mesh
bc_3D = DirichletBC(V1, u0, boundary)
# 1D submesh
bc_1D = DirichletBC(V2, u0, boundary)

# Define variational problem
# 3D
u_3D = TrialFunction(V1)
v_3D = TestFunction(V1)
# 1D
u_1D = TrialFunction(V2)
v_1D = TestFunction(V2)

f = Expression("10*exp(-(pow(x[0] - 0.5, 2) ) / 0.02)", degree=2)

a_3D = inner(grad(u_3D), grad(v_3D))*dx
L_3D = f*v_3D*dx
u_3D = Function(V1)
solve(a_3D == L_3D, u_3D, bc_3D)

a_1D = inner(grad(u_1D), grad(v_1D))*dx
L_1D = f*v_1D*dx
u_1D = Function(V2)
solve(a_1D == L_1D, u_1D, bc_1D)

# Save solution in vtk format
out_3D = File("meshview-mapping-3D1D-3Dsol.pvd")
out_3D << u_3D
out_1D = File("meshview-mapping-3D1D-1Dsol.pvd")
out_1D << u_1D



from dolfin import *

# Create mesh and define function space
mesh = UnitCubeMesh(32, 32, 32)

marker = MeshFunction("size_t", mesh, mesh.topology().dim()-2, 0)
for e in edges(mesh):
    marker[e] = 0.5 - DOLFIN_EPS < e.midpoint().z() < 0.5 + DOLFIN_EPS and 0.5 - DOLFIN_EPS < e.midpoint().y() < 0.5 + DOLFIN_EPS

submesh = MeshView.create(marker, 1)

W1 = FunctionSpace(mesh, "Lagrange", 1) ## 3D
W2 = FunctionSpace(submesh, "Lagrange", 1) ## 1D

# Define the mixed function space
V = MixedFunctionSpace( W1, W2 )

# Define Dirichlet boundary (x = 0 or x = 1)
def boundary(x):
    return x[0] < DOLFIN_EPS or x[0] > 1.0 - DOLFIN_EPS

# Define boundary condition
u0 = Constant(0.0)
# 3D mesh
bc_3D = DirichletBC(V.sub_space(0), u0, boundary)
# 1D submesh
bc_1D = DirichletBC(V.sub_space(1), u0, boundary)
bcs = [bc_3D,bc_1D]

# Define variational problem
# Use directly TrialFunction and TestFunction on the product space
(u_3D,u_1D) = TrialFunctions(V)
(v_3D,v_1D) = TestFunctions(V)

f = Expression("10*exp(-(pow(x[0] - 0.5, 2) ) / 0.02)", degree=2)

# Global weak form
a = inner(grad(u_3D), grad(v_3D))*dx + inner(grad(u_1D), grad(v_1D))*dx
L = f*v_3D*dx + f*v_1D*dx

# Solve the problem
sol = Function(V)
rtol=1e-4
print("******************************************************************")
print("NOTE : Relative tolerance for Krylov solver has been set to", rtol)
print("With default solver/preconditioner, convergence is very slow.")
print("We need to find a better configuration.")
print("******************************************************************")
solve(a == L, sol, bcs=bcs, solver_parameters={"krylov_solver":{"relative_tolerance":rtol, "maximum_iterations":10000}})

# extract components of the solution
sol_3D = sol.sub(0)
sol_1D = sol.sub(1)

## Export result
encoding = XDMFFile.Encoding.HDF5 if has_hdf5() else XDMFFile.Encoding.ASCII

out_3D = XDMFFile(MPI.comm_world, "block-assembly-3D1D-3Dsol.xdmf")
out_1D = XDMFFile(MPI.comm_world, "block-assembly-3D1D-1Dsol.xdmf")

if MPI.size(MPI.comm_world) > 1 and encoding == XDMFFile.Encoding.ASCII:
    print("XDMF file output not supported in parallel without HDF5")
else:
    out_3D.write(sol_3D, encoding)
    out_1D.write(sol_1D, encoding)




# -*- coding: utf-8 -*-
"""Utilities for getting dolfin repository paths."""

# Copyright (C) 2014-2014 Martin Sandve Alnæs
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.


import os


def find_parent_dir_with_file(filepath, filename):
    "Return the first parent directory containing a file with filename."
    # Start with given file path
    d = os.path.dirname(os.path.abspath(filepath))
    t = ''
    # Look for a particular filename
    while not os.path.isfile(os.path.join(d, filename)):
        d, t = os.path.split(d)
    # Return the directory where we found the file
    return d


def find_parent_dir_with_name(filepath, dirname):
    "Return the first parent directory called dirname."
    # Start with given file path
    d = os.path.dirname(os.path.abspath(filepath))
    t = ''
    # Look for a particular dirname
    while t != dirname:
        d, t = os.path.split(d)
    # Return the found directory
    return os.path.join(d, t)


def find_testdir(filepath):
    "Return the test/ directory filepath is contained in."
    return find_parent_dir_with_name(filepath, "test")


def find_demodir(filepath):
    "Return the demo/ directory filepath is contained in."
    return find_parent_dir_with_name(filepath, "demo")


def find_rootdir(filepath):
    """Return the root directory filepath is contained in.

    Assuming the existence of the files "INSTALL" and "AUTHORS".
    """
    a = find_parent_dir_with_file(filepath, "INSTALL")
    b = find_parent_dir_with_file(filepath, "AUTHORS")
    assert a == b
    return a


def find_cppsourcedir(filepath):
    d = find_rootdir(filepath)
    d = os.path.join(d, "dolfin")
    return d


def find_pysourcedir(filepath):
    d = find_rootdir(filepath)
    d = os.path.join(d, "site-packages")
    return d


def find_docdir(filepath):
    d = find_rootdir(filepath)
    d = os.path.join(d, "doc")
    return d


def find_benchdir(filepath):
    d = find_rootdir(filepath)
    d = os.path.join(d, "bench")
    return d



"""Unit tests for the class MeshMarkers. These unit test actually test
a bit more than that, since they also test marking of meshes and the
interaction between Mesh - MeshDomains - MeshMarkers"""

# Copyright (C) 2011 Anders Logg
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# First added:  2011-09-01
# Last changed: 2011-09-02

import pytest
from dolfin import *


def test_subdomain_marking():
    "Test setting markers from subdomains"

    # Define subdomains for 5 of the 6 faces of the unit cube
    class F0(SubDomain):
        def inside(self, x, inside):
            return near(x[0], 0.0)

    class F1(SubDomain):
        def inside(self, x, inside):
            return near(x[0], 1.0)

    class F2(SubDomain):
        def inside(self, x, inside):
            return near(x[1], 0.0)

    class F3(SubDomain):
        def inside(self, x, inside):
            return near(x[1], 1.0)

    class F4(SubDomain):
        def inside(self, x, inside):
            return near(x[2], 0.0)
    f0 = F0()
    f1 = F1()
    f2 = F2()
    f3 = F3()
    f4 = F4()

    # Apply markers to unit cube
    mesh = UnitCubeMesh(3, 3, 3)
    f0.mark_facets(mesh, 0)
    f1.mark_facets(mesh, 1)
    f2.mark_facets(mesh, 2)
    f3.mark_facets(mesh, 3)
    f4.mark_facets(mesh, 4)

    # FIXME: Add test here
    assert 0 == 0


def test_domain_markers():
    mesh = UnitSquareMesh(3, 3)
    mdoms = mesh.domains()
    ret = mdoms.set_marker((0, 2), 2)
    assert ret
    ret = mdoms.set_marker((0, 3), 2)
    assert not ret

    assert mdoms.get_marker(0, 2) == 3



from dolfin import *

# Create mesh and define function space
mesh = UnitCubeMesh(32, 32, 32)

marker = MeshFunction("size_t", mesh, mesh.topology().dim()-1, 0)
for f in facets(mesh):
    marker[f] = 0.5 - DOLFIN_EPS < f.midpoint().z() < 0.5 + DOLFIN_EPS

submesh = MeshView.create(marker, 1)

V1 = FunctionSpace(mesh, "Lagrange", 1) ## 3D
V2 = FunctionSpace(submesh, "Lagrange", 1) ## 2D

# Define Dirichlet boundary (x = 0 or x = 1)
def boundary(x):
    return x[0] < DOLFIN_EPS or x[0] > 1.0 - DOLFIN_EPS

# Define boundary condition
u0 = Constant(0.0)
# 3D mesh
bc_3D = DirichletBC(V1, u0, boundary)
# 2D submesh
bc_2D = DirichletBC(V2, u0, boundary)

# Define variational problem
# 3D
u_3D = TrialFunction(V1)
v_3D = TestFunction(V1)
# 2D
u_2D = TrialFunction(V2)
v_2D = TestFunction(V2)

f = Expression("10*exp(-(pow(x[0] - 0.5, 2) + pow(x[1] - 0.5, 2) ) / 0.02)", degree=2)

a_3D = inner(grad(u_3D), grad(v_3D))*dx
L_3D = f*v_3D*dx
u_3D = Function(V1)
solve(a_3D == L_3D, u_3D, bc_3D)

a_2D = inner(grad(u_2D), grad(v_2D))*dx
L_2D = f*v_2D*dx
u_2D = Function(V2)
solve(a_2D == L_2D, u_2D, bc_2D)

# ## Export result
# encoding = XDMFFile.Encoding.HDF5 if has_hdf5() else XDMFFile.Encoding.ASCII

# out_3D = XDMFFile(MPI.comm_world, "meshview-mapping-3D2D-3Dsol.xdmf")
# out_2D = XDMFFile(MPI.comm_world, "meshview-mapping-3D2D-2Dsol.xdmf")

# if MPI.size(MPI.comm_world) > 1 and encoding == XDMFFile.Encoding.ASCII:
#     print("XDMF file output not supported in parallel without HDF5")
# else:
#     out_3D.write(u_3D, encoding)
#     out_2D.write(u_2D, encoding)



"""Unit tests for TimeSeries"""

# Copyright (C) 2011-2014 Marie E. Rognes
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.

import pytest
from dolfin import *
import os
from dolfin_utils.test import skip_in_parallel, skip_if_not_HDF5, fixture, tempdir

@skip_in_parallel
@skip_if_not_HDF5
def test_retrieve_all_connectivities(tempdir):
    _test_retrieve(tempdir, "test_retrieve_all_connectivities")

def _test_retrieve(tempdir, basename):
    filename = os.path.join(tempdir, basename)

    times = [t/10.0 for t in range(1, 11)]

    mesh_size = (2, 2, 2)
    mesh = UnitCubeMesh(*mesh_size)
    mesh.init()
    V = FunctionSpace(mesh, "CG", 2)

    u = Function(V)
    series = TimeSeries(MPI.comm_world, filename)
    for t in times:
        u.vector()[:] = t
        series.store(u.vector(), t)
        series.store(mesh, t)

    series = TimeSeries(MPI.comm_world, filename)
    t0 = series.vector_times()[0]
    T = series.mesh_times()[-1]

    assert round(t0 - times[0], 7) == 0
    assert round(T - times[-1], 7) == 0

    # Test retreiving of mesh
    mesh_retreived = Mesh()
    series.retrieve(mesh_retreived, 0.1)

    mesh_test = mesh

    for entity in range(4):
        mesh_retreived.init(entity)
        assert mesh_retreived.topology().size(entity) == \
            mesh_test.topology().size(entity)

    for i in range(4):
        for j in range(4):
            mesh_retreived.init(i, j)
            assert mesh_retreived.topology()(i, j).size() == \
                              mesh_test.topology()(i, j).size()


@skip_in_parallel
@skip_if_not_HDF5
def test_subdirectory(tempdir):
    "Test that retrieve/store works with nonexisting subdirectory"
    filename = os.path.join(tempdir, "test_subdirectory")

    m0 = UnitSquareMesh(3, 3)

    series0 = TimeSeries(MPI.comm_world, filename)
    x0 = Vector(MPI.comm_world, 10)

    # Test storage of only one time point for the mesh
    series0.store(m0, 0.1)
    series0.store(x0, 0.1)
    series0.store(x0, 0.2)

    series1 = TimeSeries(MPI.comm_world, filename)
    m1 = Mesh()
    x1 = Vector()

    series1.retrieve(m1, 0.1)
    series1.retrieve(x1, 0.15)



"""Unit tests for intersection computation"""

# Copyright (C) 2013-2014 Anders Logg
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.

import pytest

from dolfin import intersect
from dolfin import (UnitIntervalMesh, UnitSquareMesh, UnitCubeMesh, BoxMesh, CellType)
from dolfin import Point, FunctionSpace, Expression, interpolate
from dolfin import MPI

from dolfin_utils.test import skip_in_parallel

@skip_in_parallel
def test_mesh_point_1d():
    "Test mesh-point intersection in 1D"

    point = Point(0.1)
    mesh = UnitIntervalMesh(16)

    intersection = intersect(mesh, point)

    assert intersection.intersected_cells() == [1]

@skip_in_parallel
def test_mesh_point_2d_triangle():
    "Test mesh-point intersection in 2D for triangular mesh"

    point = Point(0.1, 0.2)
    mesh = UnitSquareMesh(16, 16)

    intersection = intersect(mesh, point)

    assert intersection.intersected_cells() == [98]

@skip_in_parallel
def test_mesh_point_3d_tetrahedron():
    "Test mesh-point intersection in 3D for tetrahedral mesh"

    point = Point(0.1, 0.2, 0.3)
    mesh = UnitCubeMesh(8, 8, 8)

    intersection = intersect(mesh, point)

    assert intersection.intersected_cells() == [816]

@skip_in_parallel
@pytest.mark.xfail(strict=True, raises=RuntimeError)
def test_mesh_point_2d_quadrilateral():
    "Test mesh-point intersection in 2D for quadrilateral mesh"

    point = Point(0.1, 0.2)
    mesh = UnitSquareMesh.create(16, 16, CellType.Type.quadrilateral)

    intersection = intersect(mesh, point)

    assert intersection.intersected_cells() == [49]

@skip_in_parallel
@pytest.mark.xfail(strict=True, raises=RuntimeError)
def test_mesh_point_3d_hexahedron():
    "Test mesh-point intersection in 3D for hexahedral mesh"

    point = Point(0.1, 0.2, 0.3)
    mesh = UnitCubeMesh.create(8, 8, 8, CellType.Type.hexahedron)

    intersection = intersect(mesh, point)

    # Returns [] now, but [136] is the correct cell.
    assert intersection.intersected_cells() == [136]



"""Unit test for XDMF output of DG0"""

# Copyright (C) 2014 Chris Richardson
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.

import pytest
from dolfin import *
from dolfin_utils.test import *

ghost_mode = set_parameters_fixture("ghost_mode", ["shared_vertex", "none"])

@skip_if_not_HDF5
@xfail_with_serial_hdf5_in_parallel
def test_xdmf_cell_scalar_ghost(cd_tempdir, ghost_mode):
    n = 8
    mesh = UnitSquareMesh(n, n)
    Q = FunctionSpace(mesh, "DG", 0)
    F = Function(Q)
    E = Expression("x[0]", degree=1)
    F.interpolate(E)

    with XDMFFile(mesh.mpi_comm(), "dg0.xdmf") as xdmf:
        xdmf.write(F)

    with HDF5File(mesh.mpi_comm(), "dg0.h5", "r") as hdf:
        vec = Vector()
        hdf.read(vec, "/VisualisationVector/0", False)

    area = MPI.sum(mesh.mpi_comm(), sum(vec.get_local()))
    assert abs(n*n - area) < 1e-9



"""Unit tests for parameter library"""

# Copyright (C) 2011 Anders Logg
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.

import pytest
import os

from dolfin import *
from dolfin_utils.test import *

xfail = pytest.mark.xfail(strict=True)

@skip_in_parallel
def test_simple(tempdir):

    # Create some parameters
    p0 = Parameters("test")
    p0.add("filename", "foo.txt")
    p0.add("maxiter", 100)
    p0.add("tolerance", 0.001)
    p0.add("monitor_convergence", True)

    # Save to file
    f0 = File(os.path.join(tempdir, "test_parameters.xml"))
    f0 << p0

    # Read from file
    p1 = Parameters()
    f1 = File(os.path.join(tempdir, "test_parameters.xml"))
    f1 >> p1

    # Check values
    assert p1.name() == "test"
    assert p1["filename"] == "foo.txt"
    assert p1["maxiter"] == 100
    assert p1["tolerance"] == 0.001
    assert p1["monitor_convergence"] == True

@skip_in_parallel
def test_gzipped_simple(tempdir):

    # Create some parameters
    p0 = Parameters("test")
    p0.add("filename", "foo.txt")
    p0.add("maxiter", 100)
    p0.add("tolerance", 0.001)
    p0.add("monitor_convergence", True)

    # Save to file
    f0 = File(os.path.join(tempdir, "test_parameters.xml.gz"))
    f0 << p0

    # Read from file
    p1 = Parameters()
    f1 = File(os.path.join(tempdir, "test_parameters.xml.gz"))
    f1 >> p1

    # Check values
    assert p1.name() == "test"
    assert p1["filename"] == "foo.txt"
    assert p1["maxiter"] == 100
    assert p1["tolerance"] == 0.001
    assert p1["monitor_convergence"] == True

@skip_in_parallel
def test_nested(tempdir):

    # Create some nested parameters
    p0 = Parameters("test")
    p00 = Parameters("sub0")
    p00.add("filename", "foo.txt")
    p00.add("maxiter", 100)
    p00.add("tolerance", 0.001)
    p00.add("monitor_convergence", True)
    p0.add("foo", "bar")
    p01 = Parameters(p00);
    p01.rename("sub1");
    p0.add(p00)
    p0.add(p01)

    # Save to file
    f0 = File(os.path.join(tempdir, "test_parameters.xml"))
    f0 << p0

    # Read from file
    p1 = Parameters()
    f1 = File(os.path.join(tempdir, "test_parameters.xml"))
    f1 >> p1

    # Check values
    assert p1.name() == "test"
    assert p1["foo"] == "bar"
    assert p1["sub0"]["filename"] == "foo.txt"
    assert p1["sub0"]["maxiter"] == 100
    assert p1["sub0"]["tolerance"] == 0.001
    assert p1["sub0"]["monitor_convergence"] == True

@skip_in_parallel
def test_parameters_update(tempdir):
    p0 = parameters.copy()
    p1 = parameters.copy()
    p1.update(p0)

@skip_in_parallel
def test_nested_read_existing(tempdir):
    """Test that we can read in a nested parameter database into
       an existing (and matching) parameter database"""

    file = File(os.path.join(tempdir, "test_parameters.xml"))
    file << parameters

    p = Parameters("test")
    file >> p
    file >> p

@skip_in_parallel
def test_solver_parameters():
    "Test that global parameters are propagated to solvers"

    # Record default values so we can change back
    absolute_tolerance = parameters["krylov_solver"]["absolute_tolerance"]

    # Set global parameters
    parameters["krylov_solver"]["absolute_tolerance"] = 1.23456

    # Create solvers
    krylov_solver = KrylovSolver()

    # Check that parameters propagate to solvers
    assert krylov_solver.parameters["absolute_tolerance"] == 1.23456

    # Reset parameters so that other tests will continue to work
    if absolute_tolerance is not None:
        parameters["krylov_solver"]["absolute_tolerance"] = absolute_tolerance

def test_parse_parameters():

    def check_pset_and_dict_args(pset, pset_dict):
        for k, v in pset_dict.items():
            assert pset[k] == v

    def create_command_line_args(pset_dict):
        # Construct args like: --a=1 --b=2 --c=3
        command_line_args1 = ["--" + str(k) + "=" + str(v) for k, v in pset_dict.items()]
        # Construct args like: --a 1 --b 2 --c 3
        command_line_args2 = [str(element) for k in pset_dict for element in (k, pset_dict[k])]
        return command_line_args1, command_line_args2

    pset_dict = {
        "integer": 1,
        "double": 2.5,
        "string": "string",
        "boolean": False
    }

    pset = Parameters("test_parse")
    for k, v in pset_dict.items():
        pset.add(k, v)

    command_line_args = create_command_line_args(pset_dict)
    pset.parse([""] + command_line_args[0])
    check_pset_and_dict_args(pset, pset_dict)
    pset.parse([""] + command_line_args[1])
    check_pset_and_dict_args(pset, pset_dict)

    changed_pset_dict = {
        "integer": 2,
        "double": 4.5,
        "string": "another string",
        "boolean": True
    }

    changed_command_line_args = create_command_line_args(changed_pset_dict)
    pset.parse([""] + changed_command_line_args[0])
    check_pset_and_dict_args(pset, changed_pset_dict)
    pset.parse([""] + changed_command_line_args[1])
    check_pset_and_dict_args(pset, changed_pset_dict)

@pytest.mark.parametrize('arg_factory', [pytest.param(True, marks=xfail),
                                         pytest.param(1, marks=xfail),
                                         pytest.param(1., marks=xfail),
                                         pytest.param("string", marks=xfail),
                                         pytest.param("none", marks=xfail),
                                         pytest.param(parameters, marks=xfail)
                                         ])
def test_wrong_parameters_setitem(arg_factory):
    value = arg_factory
    parameters["misplelled"] = arg_factory

@xfail
def test_wrong_parameters_getitem(arg_factory):
    parameters["misplelled"]



#!/usr/bin/env py.test

"""Unit tests for MixedAssembler"""

# Copyright (C) 2011 Johan Hake
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# Modified by Cecile Daversin-Catty 2018

import pytest
import numpy as np
from dolfin import *
from ufl.log import UFLException

from dolfin_utils.test import skip_in_parallel, fixture

@fixture
def one_element(): # Reference element
    mesh = UnitSquareMesh(1, 1, "left")
    cell_f = MeshFunction('size_t', mesh, mesh.topology().dim(), 0)
    cell_f[0] = 1
    mesh = SubMesh(mesh, cell_f, 1)

    marker = MeshFunction('size_t', mesh, mesh.topology().dim()-1, 0)
    for f in facets(mesh):
        marker[f] = 0.0 - DOLFIN_EPS < f.midpoint().x() < 0.0 + DOLFIN_EPS

    return (mesh, marker)

@fixture
def two_elements():
    mesh = UnitSquareMesh(1, 1, "left")

    marker = MeshFunction('size_t', mesh, mesh.topology().dim()-1, 0)
    for f in facets(mesh):
        marker[f] = 0.5 - DOLFIN_EPS < f.midpoint().x() < 0.5 + DOLFIN_EPS and 0.5 - DOLFIN_EPS < f.midpoint().y() < 0.5 + DOLFIN_EPS

    return (mesh, marker)

@fixture
def two_elements_with_interface():
    mesh = UnitSquareMesh(1, 1)

    markerc = MeshFunction('size_t', mesh, mesh.topology().dim(), 0)
    markerf = MeshFunction('size_t', mesh, mesh.topology().dim()-1, 0)
    for c in cells(mesh):
        markerc[c] = c.midpoint().y() > c.midpoint().x()
    for f in facets(mesh):
        markerf[f] = abs(f.midpoint().x() - f.midpoint().y()) < 1e-10

    return (markerc, markerf)

@fixture
def unit_marker_2D2D():
    n = 20
    square = UnitSquareMesh(n, n)
    marker = MeshFunction("size_t", square, square.topology().dim(), 0)
    for c in cells(square):
        marker[c] = c.midpoint().x() < 0.5
    return marker

@fixture
def unit_marker_3D2D():
    n = 20
    cube = UnitCubeMesh(n, n, n)
    marker = MeshFunction("size_t", cube, cube.topology().dim() - 1, 0)
    for f in facets(cube):
        marker[f] = 0.5 - DOLFIN_EPS < f.midpoint().z() < 0.5 + DOLFIN_EPS
    return marker

@fixture
def unit_marker_ext():
    square = UnitSquareMesh(1, 1)
    marker = MeshFunction("size_t", square, square.topology().dim()-1, 0)
    for f in facets(square):
        marker[f] = abs(f.midpoint().y()) < 1e-10
    return marker

def meshview(marker, i):
    return MeshView.create(marker, i)

def space(mesh):
    return FunctionSpace(mesh, "Lagrange", 1)

def a(u,v):
    return inner(grad(u), grad(v))*dx

def L(f,v):
    return f*v*dx

def boundary(x):
    return x[0] < DOLFIN_EPS or x[0] > 1.0 - DOLFIN_EPS

def boundary1(x):
    return x[0] > 1.0 - DOLFIN_EPS

def boundary2(x):
    return x[0] < DOLFIN_EPS

def params():
    return dict({"linear_solver":"direct"})

@skip_in_parallel
def test_mixed_assembly(one_element, two_elements):
    """Off-diagonal blocks assembly"""

    ## Test for a range of various FE types
    def _check_scalar(case, order):
        mesh = case[0]
        submesh = MeshView.create(case[1], 1)

        V = FunctionSpace(mesh, 'CG', order[0])
        Q = FunctionSpace(submesh, 'CG', order[1])
        W = MixedFunctionSpace(V, Q)

        f = Expression('x[0]+x[1]', degree=3)
        g = Expression('x[0]-x[1]', degree=3)

        (u, p) = TrialFunctions(W)
        (v, q) = TestFunctions(W)

        dx_ = Measure('dx', domain=W.sub_space(1).mesh())
        ## Reference value
        ref = assemble(inner(f, g)*dx_)

        trace_form = inner(u, q)*dx_ + inner(u, v)*dx + inner(p, v)*dx_ + inner(p, q)*dx_
        rhs = inner(Constant(0), v)*dx + inner(Constant(0), q)*dx_

        AA, bb, _ = assemble_mixed_system(trace_form == rhs, Function(W))

        T = AA[2]
        Tt = AA[1]

        v = interpolate(f, V)
        q = interpolate(g, Q)

        Tv = Function(Q).vector()
        T.mult(v.vector(), Tv)
        qT = Function(V).vector()
        Tt.mult(q.vector(), qT)

        vqT = v.vector().inner(qT)
        qTv = q.vector().inner(Tv)

        assert(abs(vqT - ref) <= 1e-12)
        assert(abs(qTv - ref) <= 1e-12)
        assert(abs(qTv - vqT) <= 1e-12)
        
    def _check_vectorial(case, order):
        mesh = case[0]
        submesh = MeshView.create(case[1], 1)

        V = VectorFunctionSpace(mesh, 'CG', order[0])
        Q = VectorFunctionSpace(submesh, 'CG', order[1])
        W = MixedFunctionSpace(V, Q)

        f = Expression(('x[0]+x[1]', 'x[0]-x[1]'), degree=3)
        g = Expression(('x[0]+3*x[1]', 'x[0]-2*x[1]'), degree=3)

        (u, p) = TrialFunctions(W)
        (v, q) = TestFunctions(W)
        dx_ = Measure('dx', domain=W.sub_space(1).mesh())
        ## Reference value
        ref = assemble(inner(f, g)*dx_)

        trace_form = inner(u, q)*dx_ + inner(u, v)*dx + inner(p, v)*dx_ + inner(p, q)*dx_
        rhs = inner(Constant((0, 0)), v)*dx + inner(Constant((0, 0)), q)*dx_

        AA, bb, _ = assemble_mixed_system(trace_form == rhs, Function(W))

        T = AA[2]
        Tt = AA[1]

        v = interpolate(f, V)
        q = interpolate(g, Q)

        Tv = Function(Q).vector()
        T.mult(v.vector(), Tv)
        qT = Function(V).vector()
        Tt.mult(q.vector(), qT)

        vqT = v.vector().inner(qT)
        qTv = q.vector().inner(Tv)

        assert(abs(vqT - ref) <= 1e-12)
        assert(abs(qTv - ref) <= 1e-12)
        assert(abs(qTv - vqT) <= 1e-12)

    # Scalar case - One element + Lagrange mult on exterior boundary
    _check_scalar(one_element, (1,2)) # CG1 - CG2
    _check_scalar(one_element, (2,1)) # CG2 - CG1
    # Scalar case - Two elements + Lagrange mult on interior facet
    _check_scalar(two_elements, (1,2)) # CG1 - CG2
    _check_scalar(two_elements, (2,1)) # CG2 - CG1
    # Vectorial case - One element + Lagrange mult on exterior boundary
    _check_vectorial(one_element, (1,2)) # CG1 - CG2
    _check_vectorial(one_element, (2,1)) # CG2 - CG1
    # Vectorial case - Two elements + Lagrange mult on interior facet
    _check_vectorial(two_elements, (1,2)) # CG1 - CG2
    _check_vectorial(two_elements, (2,1)) # CG2 - CG1

@skip_in_parallel
def test_mixed_assembly_interface(two_elements_with_interface):
    """Off-diagonal blocks assembly"""

    ## Test for a range of various FE types
    def _check_scalar(case, order):
        mesh0 = MeshView.create(case[0], 0)
        mesh1 = MeshView.create(case[0], 1)
        submesh = MeshView.create(case[1], 1)

        V0 = FunctionSpace(mesh0, 'CG', order[0])
        V1 = FunctionSpace(mesh1, 'CG', order[1])
        Q = FunctionSpace(submesh, 'CG', order[2])
        W = MixedFunctionSpace(V0, V1, Q)

        f = Expression('x[0]+x[1]', degree=3)
        g = Expression('x[0]-x[1]', degree=3)

        (u0, u1, p) = TrialFunctions(W)
        (v0, v1, q) = TestFunctions(W)

        dx_ = Measure('dx', domain=W.sub_space(2).mesh())
        # Reference value
        ref_fg = assemble(inner(f, g)*dx_)
        ref_ff = assemble(inner(f, f)*dx_)

        # Diagonal blocks
        trace_form = inner(u0, v0)*dx + inner(u1, v1)*dx + inner(p, q)*dx_
        # Mixed-dimensional coupling
        trace_form += inner(u0, q)*dx_ + inner(u1, q)*dx_ + inner(p, v0)*dx_ + inner(p, v1)*dx_
        # Integration of common
        trace_form += inner(u0, v1)*dx_ + inner(u1,v0)*dx_
        rhs = inner(Constant(0), v0)*dx + inner(Constant(0), v1)*dx + inner(Constant(0), q)*dx_

        AA, bb, _ = assemble_mixed_system(trace_form == rhs, Function(W))

        # Mixed-dimensional coupling
        T0 = AA[6] # u0*q
        T1 = AA[7] # u1*q
        Tt0 = AA[2] # p*v0
        Tt1 = AA[5] # p*v1
        # Integration of common interface
        I = AA[3] # u0*v1
        It = AA[1] # u1*v0

        v0 = interpolate(f, W.sub_space(0))
        v1 = interpolate(f, W.sub_space(1))
        q = interpolate(g, W.sub_space(2))

        Tv0 = Function(Q).vector()
        Tv1 = Function(Q).vector()
        T0.mult(v0.vector(), Tv0)
        T1.mult(v1.vector(), Tv1)
        qT0 = Function(V0).vector()
        qT1 = Function(V1).vector()
        Tt0.mult(q.vector(), qT0)
        Tt1.mult(q.vector(), qT1)

        Iv0 = Function(V1).vector()
        Iv1 = Function(V0).vector()
        I.mult(v0.vector(), Iv0)
        It.mult(v1.vector(), Iv1)

        v0qT0 = v0.vector().inner(qT0)
        v1qT1 = v1.vector().inner(qT1)
        qTv0 = q.vector().inner(Tv0)
        qTv1 = q.vector().inner(Tv1)

        v1Iv0 = v1.vector().inner(Iv0)
        v0Iv1 = v0.vector().inner(Iv1)

        assert(abs(v0qT0 - ref_fg) <= 1e-12)
        assert(abs(v1qT1 - ref_fg) <= 1e-12)
        assert(abs(qTv0 - ref_fg) <= 1e-12)
        assert(abs(qTv1 - ref_fg) <= 1e-12)
        assert(abs(qTv0 - v0qT0) <= 1e-12)
        assert(abs(qTv1 - v1qT1) <= 1e-12)

        assert(abs(v1Iv0 - ref_ff) <= 1e-12)
        assert(abs(v0Iv1 - ref_ff) <= 1e-12)

        # print("[scalar] v1Iv0 = ", v1Iv0, " while ref_ff = ", ref_ff)
        # print("[scalar] v0Iv1 = ", v0Iv1, " while ref_ff = ", ref_ff)


    def _check_vectorial(case, order):
        mesh0 = MeshView.create(case[0], 0)
        mesh1 = MeshView.create(case[0], 1)
        submesh = MeshView.create(case[1], 1)

        V0 = VectorFunctionSpace(mesh0, 'CG', order[0])
        V1 = VectorFunctionSpace(mesh1, 'CG', order[1])
        Q = VectorFunctionSpace(submesh, 'CG', order[2])
        W = MixedFunctionSpace(V0, V1, Q)

        f = Expression(('x[0]+x[1]', 'x[0]-x[1]'), degree=3)
        g = Expression(('x[0]+3*x[1]', 'x[0]-2*x[1]'), degree=3)

        (u0, u1, p) = TrialFunctions(W)
        (v0, v1, q) = TestFunctions(W)
        dx_ = Measure('dx', domain=W.sub_space(2).mesh())
        ## Reference value
        ref_fg = assemble(inner(f, g)*dx_)
        ref_ff = assemble(inner(f, f)*dx_)

        # Diagonal blocks
        trace_form = inner(u0, v0)*dx + inner(u1, v1)*dx + inner(p, q)*dx_
        # Mixed-dimensional coupling
        trace_form += inner(u0, q)*dx_ + inner(u1, q)*dx_ + inner(p, v0)*dx_ + inner(p, v1)*dx_
        # Integration of common interface
        trace_form += inner(u0, v1)*dx_ + inner(u1, v0)*dx_
        rhs = inner(Constant((0, 0)), v0)*dx + inner(Constant((0, 0)), v1)*dx + inner(Constant((0, 0)), q)*dx_

        AA, bb, _ = assemble_mixed_system(trace_form == rhs, Function(W))

        # Mixed-dimensional coupling
        T0 = AA[6]
        T1 = AA[7]
        Tt0 = AA[2]
        Tt1 = AA[5]
        # Integration of common interface
        I = AA[3] # u0*v1
        It = AA[1] # u1*v0

        v0 = interpolate(f, W.sub_space(0))
        v1 = interpolate(f, W.sub_space(1))
        q = interpolate(g, W.sub_space(2))

        Tv0 = Function(Q).vector()
        Tv1 = Function(Q).vector()
        T0.mult(v0.vector(), Tv0)
        T1.mult(v1.vector(), Tv1)
        qT0 = Function(V0).vector()
        qT1 = Function(V1).vector()
        Tt0.mult(q.vector(), qT0)
        Tt1.mult(q.vector(), qT1)

        Iv0 = Function(V1).vector()
        Iv1 = Function(V0).vector()
        I.mult(v0.vector(), Iv0)
        It.mult(v1.vector(), Iv1)

        v0qT0 = v0.vector().inner(qT0)
        v1qT1 = v1.vector().inner(qT1)
        qTv0 = q.vector().inner(Tv0)
        qTv1 = q.vector().inner(Tv1)

        v1Iv0 = v1.vector().inner(Iv0)
        v0Iv1 = v0.vector().inner(Iv1)

        assert(abs(v0qT0 - ref_fg) <= 1e-12)
        assert(abs(v1qT1 - ref_fg) <= 1e-12)
        assert(abs(qTv0 - ref_fg) <= 1e-12)
        assert(abs(qTv1 - ref_fg) <= 1e-12)
        assert(abs(qTv0 - v0qT0) <= 1e-12)
        assert(abs(qTv1 - v1qT1) <= 1e-12)

        assert(abs(v1Iv0 - ref_ff) <= 1e-12)
        assert(abs(v0Iv1 - ref_ff) <= 1e-12)

        # print("[vectorial] v1Iv0 = ", v1Iv0, " while ref_ff = ", ref_ff)
        # print("[vectorial] v0Iv1 = ", v0Iv1, " while ref_ff = ", ref_ff)

    # Scalar case - Two elements + Lagrange mult on interior facet
    _check_scalar(two_elements_with_interface, (1,1,1)) # CG1 - CG1 - CG2
    _check_scalar(two_elements_with_interface, (2,2,1)) # CG2 - CG2 - CG1
    _check_scalar(two_elements_with_interface, (1,1,2)) # CG1 - CG1- CG2
    _check_scalar(two_elements_with_interface, (2,2,2)) # CG2 - CG2- CG1
    _check_scalar(two_elements_with_interface, (1,1,2)) # CG2 - CG1- CG1
    _check_scalar(two_elements_with_interface, (1,2,1)) # CG1 - CG2- CG2
    _check_scalar(two_elements_with_interface, (2,1,1)) # CG1 - CG2- CG2
    # Vectorial case - Two elements + Lagrange mult on interior facet
    _check_vectorial(two_elements_with_interface, (1,1,2)) # CG1 - CG1 - CG2
    _check_vectorial(two_elements_with_interface, (2,2,1)) # CG2 - CG2 - CG1
    _check_vectorial(two_elements_with_interface, (1,1,2)) # CG1 - CG1- CG2
    _check_vectorial(two_elements_with_interface, (2,2,1)) # CG2 - CG2- CG1
    _check_vectorial(two_elements_with_interface, (2,1,1)) # CG2 - CG1- CG1
    _check_vectorial(two_elements_with_interface, (1,2,1)) # CG1 - CG2- CG2

@skip_in_parallel
def test_mixed_assembly_diag(unit_marker_2D2D, unit_marker_3D2D):
    def _compare_solutions(marker, boundaries):
        # Meshes
        _mesh1 = meshview(marker,0)
        _mesh2 = meshview(marker,1)
        # Spaces
        _space1 = space(_mesh1)
        _space2 = space(_mesh2)
        _space_mixed = MixedFunctionSpace(_space1, _space2)
        # Trial functions
        _u1 = TrialFunction(_space1)
        _u2 = TrialFunction(_space2)
        (_u1_m, _u2_m) = TrialFunctions(_space_mixed)
        # Test functions
        _v1 = TestFunction(_space1)
        _v2 = TestFunction(_space2)
        (_v1_m, _v2_m) = TestFunctions(_space_mixed)
        # Bilinear forms
        _a1 = a(_u1,_v1)
        _a2 = a(_u2,_v2)
        _am = a(_u1_m, _v1_m) + a(_u2_m, _v2_m)
        # Linear forms
        f = Expression("10*exp(-(pow(x[0] - 0.5, 2) + pow(x[1] - 0.5, 2)) / 0.02)", degree=2)
        _L1 = L(f,_v1)
        _L2 = L(f,_v2)
        _Lm = L(f,_v1_m) + L(f,_v2_m)
        # Solution
        sol1 = Function(_space1)
        sol2 = Function(_space2)
        sol = Function(_space_mixed)
        # BCs
        bc1 = DirichletBC(_space1, Constant(0.0), boundaries[0])
        bc2 = DirichletBC(_space2, Constant(0.0), boundaries[1])
        # Resolution
        solve(_a1 == _L1, sol1, bcs=bc1)
        solve(_a2 == _L2, sol2, bcs=bc2)
        solve(_am == _Lm, sol, bcs=[bc1, bc2], solver_parameters=params())

        sol1_m = sol.sub(0,deepcopy=True)
        sol2_m = sol.sub(1,deepcopy=True)
        assert len(sol1.vector()) == len(sol1_m.vector())
        for i in range(len(sol1.vector())):
            assert abs(sol1.vector()[i] - sol1_m.vector()[i]) < 1e-10

        assert len(sol2.vector()) == len(sol2_m.vector())
        for i in range(len(sol2.vector())):
            assert abs(sol2.vector()[i] - sol2_m.vector()[i]) < 1e-10


    _compare_solutions(unit_marker_2D2D, [boundary1, boundary2])
    _compare_solutions(unit_marker_3D2D, [boundary, boundary])


@skip_in_parallel
def test_mixed_assembly_rank0(unit_marker_ext):
    def _compare_solutions(marker):
        # Meshes
        mesh = marker.mesh() # The initial parent mesh
        submesh = meshview(marker,1)

        Vh0 = FunctionSpace(mesh, "Lagrange", 1)
        Vh1 = FunctionSpace(submesh, "Lagrange", 1)

        v0 = Function(Vh0)
        m0 = Function(Vh0)
        m1 = Function(Vh1)

        v0.vector()[:] = np.random.rand(Vh0.dim())
        m0.vector()[:] = np.ones(Vh0.dim())
        m1.vector()[:] = np.ones(Vh1.dim())

        ds = Measure("ds", domain=mesh, subdomain_data=marker)
        f0 = v0 * m0 * ds(1)

        ds1 = Measure("dx", domain=submesh)
        f1 = v0 * m1 * ds1

        assert abs(assemble(f0) - assemble_mixed(f1)) < 1e-10

    _compare_solutions(unit_marker_ext)



# -*- coding: utf-8 -*-

# Copyright (C) 2011-2017 Anders Logg and Garth N. Wells
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.

from ufl.form import sub_forms_by_domain

import dolfin.cpp as cpp
import dolfin.fem.solving
from dolfin.fem.form import Form


class LinearVariationalProblem(cpp.fem.LinearVariationalProblem):

    def __init__(self, a, L, u, bcs=None, form_compiler_parameters=None):
        """Create linear variational problem a(u, v) = L(v).

        An optional argument bcs may be passed to specify boundary
        conditions.

        Another optional argument form_compiler_parameters may be
        specified to pass parameters to the form compiler.

        """

        if bcs is None:
            bcs = []
        elif not isinstance(bcs, (list, tuple)):
            bcs = [bcs]

        # Store input UFL forms and solution Function
        self.a_ufl = a
        self.L_ufl = L
        self.u_ufl = u

        # Store form compiler parameters
        form_compiler_parameters = form_compiler_parameters or {}
        self.form_compiler_parameters = form_compiler_parameters

        # Wrap forms (and check if linear form L is empty)
        if L.empty():
            L = cpp.fem.Form(1, 0)
        else:
            L = Form(L, form_compiler_parameters=form_compiler_parameters)
        a = Form(a, form_compiler_parameters=form_compiler_parameters)

        # Initialize C++ base class
        cpp.fem.LinearVariationalProblem.__init__(self, a, L, u._cpp_object, bcs)


class MixedLinearVariationalProblem(cpp.fem.MixedLinearVariationalProblem):

    def __init__(self, a, L, u, bcs=None, form_compiler_parameters=None):
        """Create mixed linear variational problem a(u, v) = L(v).

        An optional argument bcs may be passed to specify boundary
        conditions.

        Another optional argument form_compiler_parameters may be
        specified to pass parameters to the form compiler.

        """

        # Extract and check arguments (u is a list of Function)
        u_comps = [u[i]._cpp_object for i in range(len(u))]
        bcs = dolfin.fem.solving._extract_bcs(bcs)

        # Store form compiler parameters
        form_compiler_parameters = form_compiler_parameters or {}
        self.form_compiler_parameters = form_compiler_parameters

        # Update rhs if we don't have a consistent number of blocks
        if len(L) != len(u):
            L_tmp = [None for i in range(len(u))]
            for Li in L:
                L_tmp[Li.arguments()[0].part()] = Li
            L = L_tmp

        # Check number of blocks in lhs, rhs are consistent
        assert (len(a) == len(u) * len(u))
        assert (len(L) == len(u))

        # Create list of forms/blocks
        a_list = list()
        L_list = list()
        for Li in L:
            if Li is None:
                L_list.append([cpp.fem.Form(1, 0)])  # single-elt list
            elif Li.empty():
                L_list.append([cpp.fem.Form(1, 0)])  # single-elt list
            else:
                Ls = []  # List of Li subforms
                for Lsub in sub_forms_by_domain(Li):
                    if Lsub is None:
                        Ls.append(cpp.fem.Form(1, 0))
                    elif Lsub.empty():
                        Ls.append(cpp.fem.Form(1, 0))
                    else:
                        Ls.append(Form(Lsub, form_compiler_parameters=form_compiler_parameters))
                L_list.append(Ls)

        for ai in a:
            if ai is None:
                a_list.append([cpp.fem.Form(2, 0)])
            else:
                As = []
                for Asub in sub_forms_by_domain(ai):
                    As.append(Form(Asub, form_compiler_parameters=form_compiler_parameters))
                a_list.append(As)

        # Initialize C++ base class
        cpp.fem.MixedLinearVariationalProblem.__init__(self, a_list, L_list, u_comps, bcs)


class NonlinearVariationalProblem(cpp.fem.NonlinearVariationalProblem):

    def __init__(self, F, u, bcs=None, J=None, form_compiler_parameters=None):
        """Create nonlinear variational problem F(u; v) = 0.

        Optional arguments bcs and J may be passed to specify boundary
        conditions and the Jacobian J = dF/du.

        Another optional argument form_compiler_parameters may be
        specified to pass parameters to the form compiler.

        """

        if bcs is None:
            bcs = []
        elif not isinstance(bcs, (list, tuple)):
            bcs = [bcs]

        # Store input UFL forms and solution Function
        self.F_ufl = F
        self.J_ufl = J
        self.u_ufl = u

        # Store form compiler parameters
        form_compiler_parameters = form_compiler_parameters or {}
        self.form_compiler_parameters = form_compiler_parameters

        # Wrap forms
        F = Form(F, form_compiler_parameters=form_compiler_parameters)
        if J is not None:
            J = Form(J, form_compiler_parameters=form_compiler_parameters)

        # Initialize C++ base class
        cpp.fem.NonlinearVariationalProblem.__init__(self, F, u._cpp_object, bcs, J)


class MixedNonlinearVariationalProblem(cpp.fem.MixedNonlinearVariationalProblem):

    def __init__(self, F, u, bcs=None, J=None, form_compiler_parameters=None):
        """Create nonlinear variational problem F(u; v) = 0.

        An optional argument bcs may be passed to specify boundary
        conditions.

        Another optional argument form_compiler_parameters may be
        specified to pass parameters to the form compiler.

        """

        # Extract and check arguments (u is a list of Function)
        u_comps = [u[i]._cpp_object for i in range(len(u))]
        bcs = dolfin.fem.solving._extract_bcs(bcs)

        # Store form compiler parameters
        form_compiler_parameters = form_compiler_parameters or {}
        self.form_compiler_parameters = form_compiler_parameters

        # Store input UFL forms and solution Function
        self.F_ufl = F
        self.J_ufl = J
        self.u_ufl = u

        # Update rhs if we don't have a consistent number of blocks
        if len(F) != len(u):
            F_tmp = [None for i in range(len(u))]
            for Fi in F:
                F_tmp[Fi.arguments()[0].part()] = Fi
            F = F_tmp

        # Check number of blocks in the residual and solution are coherent
        assert (len(J) == len(u) * len(u))
        assert (len(F) == len(u))

        # Create list of forms/blocks
        F_list = list()
        for Fi in F:
            if Fi is None:
                F_list.append([cpp.fem.Form(1, 0)])
            elif Fi.empty():
                F_list.append([cpp.fem.Form(1, 0)])  # single-elt list
            else:
                Fs = []
                for Fsub in sub_forms_by_domain(Fi):
                    if Fsub is None:
                        Fs.append(cpp.fem.Form(1, 0))
                    elif Fsub.empty():
                        Fs.append(cpp.fem.Form(1, 0))
                    else:
                        Fs.append(Form(Fsub, form_compiler_parameters=form_compiler_parameters))
                F_list.append(Fs)

        J_list = None
        if J is not None:
            J_list = list()
            for Ji in J:
                if Ji is None:
                    J_list.append([cpp.fem.Form(2, 0)])
                elif Ji.empty():
                    J_list.append([cpp.fem.Form(2, 0)])
                else:
                    Js = []
                    for Jsub in sub_forms_by_domain(Ji):
                        Js.append(Form(Jsub, form_compiler_parameters=form_compiler_parameters))
                    J_list.append(Js)

        # Initialize C++ base class
        cpp.fem.MixedNonlinearVariationalProblem.__init__(self, F_list, u_comps, bcs, J_list)



# -*- coding: utf-8 -*-
"""The multistage module of dolfin"""

from dolfin.multistage import multistagescheme
from dolfin.multistage import multistagesolvers
from dolfin.multistage import rushlarsenschemes

from .multistagescheme import *  # noqa
from .multistagesolvers import *  # noqa
from .rushlarsenschemes import *  # noqa

# NOTE: The automatic documentation system in DOLFIN requires to _not_
# define classes or functions within this file. Use separate modules
# for that purpose.

__all__ = multistagescheme.__all__ + multistagesolvers.__all__ + \
    rushlarsenschemes.__all__




import dolfin.cpp as cpp

_meshvaluecollection_types = {"bool": cpp.mesh.MeshValueCollection_bool,
                              "size_t": cpp.mesh.MeshValueCollection_sizet,
                              "int": cpp.mesh.MeshValueCollection_int,
                              "double": cpp.mesh.MeshValueCollection_double}


class MeshValueCollection(object):
    def __new__(cls, value_type, mesh, dim=None):
        if value_type not in _meshvaluecollection_types.keys():
            raise KeyError("MeshValueCollection type not recognised")
        mvc = _meshvaluecollection_types[value_type]
        if dim is not None:
            return mvc(mesh, dim)
        else:
            return mvc(mesh)



"""Unit tests for MPI facilities"""

# Copyright (C) 2017 Garth N. Wells
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.

import dolfin
from dolfin import MPI
from dolfin_utils.test import skip_if_not_petsc4py


def test_mpi_comm_wrapper():
    """
    Test MPICommWrapper <-> mpi4py.MPI.Comm conversion
    """
    if dolfin.has_mpi4py():
        from mpi4py import MPI
        w1 = MPI.COMM_WORLD
    else:
        w1 = dolfin.MPI.comm_world

    m = dolfin.UnitSquareMesh(w1, 4, 4)
    w2 = m.mpi_comm()

    if dolfin.has_mpi4py():
        assert isinstance(w1, MPI.Comm)
        assert isinstance(w2, MPI.Comm)
    else:
        assert isinstance(w1, dolfin.cpp.MPICommWrapper)
        assert isinstance(w2, dolfin.cpp.MPICommWrapper)



"""Unit tests for FunctionSpace with constrained domain"""

# Copyright (C) 2012-2014 Garth N. Wells
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# Modified by Mikael Mortensen 2013

import pytest
import numpy
from dolfin import *


class PeriodicBoundary2(SubDomain):
    def __init__(self, tolerance=DOLFIN_EPS):
        SubDomain.__init__(self, tolerance)
        self.tol = tolerance

    def inside(self, x, on_boundary):
        return bool(x[0] < self.tol and x[0] > -self.tol and on_boundary)

    def map(self, x, y):
        y[0] = x[0] - 1.0
        y[1] = x[1]


class PeriodicBoundary3(SubDomain):
    def inside(self, x, on_boundary):
        return bool(x[0] < DOLFIN_EPS and x[0] > -DOLFIN_EPS and on_boundary)

    def map(self, x, y):
        y[0] = x[0] - 1.0
        y[1] = x[1]
        y[2] = x[2]


def test_instantiation():
    """ A rudimentary test for instantiation"""

    sub_domain = PeriodicBoundary3()
    mesh = UnitCubeMesh(8, 8, 8)
    V = FunctionSpace(mesh, "CG", 1, constrained_domain=sub_domain)


def test_instantiation_mixed_element():
    """A rudimentary test for instantiation with mixed elements"""

    pbc = PeriodicBoundary2()
    mesh = UnitSquareMesh(8, 8)
    P1 = FiniteElement("Lagrange", mesh.ufl_cell(), 1)
    VV = FunctionSpace(mesh, P1*P1, constrained_domain=pbc)


def test_instantiation_mixed_element_real():
    """A rudimentary test for instantiation with mixed elements that
    include a real space
    """

    pbc = PeriodicBoundary2()
    mesh = UnitSquareMesh(8, 8)
    P1 = FiniteElement("Lagrange", mesh.ufl_cell(), 1)
    R = FiniteElement("Real", mesh.ufl_cell(), 0)
    VV = FunctionSpace(mesh, P1*R, constrained_domain=pbc)
    VV = FunctionSpace(mesh, R*P1, constrained_domain=pbc)


def test_instantiation_no_vertex_element_2D():
    """ A rudimentary test for instantiation for element that does
    not require number of vertices (2D)"""

    pbc = PeriodicBoundary2()
    mesh = UnitSquareMesh(8, 8)
    V = FunctionSpace(mesh, "BDM", 1, constrained_domain=pbc)


def test_instantiation_no_vertex_element_3D():
    """ A rudimentary test for instantiation for element that does
    not require number of vertices (3D)"""

    pbc = PeriodicBoundary3()
    mesh = UnitCubeMesh(8, 8, 9)
    V = FunctionSpace(mesh, "BDM", 1, constrained_domain=pbc)


def test_director_lifetime():
    """Test for problems with objects with directors going out
    of scope"""

    mesh = UnitSquareMesh(8, 8)
    V = FunctionSpace(mesh, "Lagrange", 1,
                      constrained_domain=PeriodicBoundary2())


def test_tolerance():
    """Test tolerance for matching periodic mesh entities"""
    shift = 0.0001
    mesh = UnitSquareMesh(8, 8)

    # Randomly perturb mesh vertex coordinates
    mesh_perturb = Mesh(mesh)
    import random
    for x in mesh_perturb.coordinates():
        x[0] += random.uniform(-shift, shift)
        x[1] += random.uniform(-shift, shift)

    pbc = PeriodicBoundary2()
    pbc_tol = PeriodicBoundary2(2*shift)

    for dim in range(mesh.geometry().dim()):
        periodic_pairs = PeriodicBoundaryComputation.compute_periodic_pairs(mesh, pbc, dim)
        num_periodic_pairs0 = len(periodic_pairs)

        periodic_pairs = PeriodicBoundaryComputation.compute_periodic_pairs(mesh_perturb,
                                                                            pbc_tol, dim)
        num_periodic_pairs1 = len(periodic_pairs)
        assert num_periodic_pairs0 == num_periodic_pairs1


def test_solution():
    """Test periodic constrained domain by checking solution to a PDE."""

    # Create mesh and constrained FunctionSpace
    mesh = UnitSquareMesh(8, 8)
    pbc = PeriodicBoundary2()
    V = FunctionSpace(mesh, "Lagrange", 1, constrained_domain=pbc)

    class DirichletBoundary(SubDomain):
        def inside(self, x, on_boundary):
            return bool((x[1] < DOLFIN_EPS or x[1] > (1.0 - DOLFIN_EPS)) and
                        on_boundary)

    # Dirichlet boundary condition
    dirichlet_boundary = DirichletBoundary()
    bc0 = DirichletBC(V, 0.0, dirichlet_boundary)
    bcs = [bc0]

    # Define variational problem, linear formulation
    u, v = TrialFunction(V), TestFunction(V)
    f = Expression("sin(x[0])", degree=2)
    a = dot(grad(u), grad(v))*dx
    L = f*v*dx

    # Compute solution
    u = Function(V)
    solve(a == L, u, bcs)

    assert round(u.vector().norm("l2") - 0.3368694028630991, 10) == 0

    # Define variational problem, nonlinear formulation
    u, v = Function(V), TestFunction(V)
    f = Expression("sin(x[0])", degree=2)
    a = dot(grad(u), grad(v))*dx
    L = f*v*dx
    F = a - L

    # Compute solution
    solve(F == 0, u, bcs)

    assert round(u.vector().norm("l2") - 0.3368694028630991, 10) == 0



# very rough demo to test out ghost cells
# run with mpirun
#

from dolfin import *
import numpy as np
import sys, os

try:
    import matplotlib.pyplot as plt
    from matplotlib.collections import PolyCollection
except ImportError:
    print("This demo requires matplotlib! Bye.")
    exit()

parameters["ghost_mode"] = "shared_vertex"
#parameters["ghost_mode"] = "shared_facet"
#parameters["ghost_mode"] = "None"
parameters["reorder_cells_gps"] = True
parameters["reorder_vertices_gps"] = True

n = 0

if(len(sys.argv) == 2):
    try:
        n = int(sys.argv[1])
    except:
        n = 0

if(MPI.size(MPI.comm_world) == 1):
    print("Only works with MPI")
    quit()

#parameters["mesh_partitioner"] = "ParMETIS"

mesh = UnitSquareMesh(8, 8)
# mesh = refine(M)

shared_vertices = np.fromiter(mesh.topology().shared_entities(0).keys(), dtype='uintc')
shared_cells = mesh.topology().shared_entities(mesh.topology().dim())

num_regular_vertices = mesh.topology().ghost_offset(0)

ghost_vertices = np.arange(num_regular_vertices, mesh.topology().size(0))

verts_note = []
if (n == 0):
    for k,val in mesh.topology().shared_entities(0).items():
        vtx = Vertex(mesh, k)
        verts_note.append( (vtx.point().x(), vtx.point().y(), " "+str(val)) )
elif (n == 1):
    for i in range(mesh.num_vertices()):
        vtx = Vertex(mesh, i)
        val = vtx.global_index()
        verts_note.append( (vtx.point().x(), vtx.point().y(), " "+str(val)) )
else:
    for i in range(mesh.num_vertices()):
        vtx = Vertex(mesh, i)
        val = vtx.index()
        verts_note.append( (vtx.point().x(), vtx.point().y(), " "+str(val)) )

x,y = mesh.coordinates().transpose()

rank = MPI.rank(mesh.mpi_comm())

cell_ownership = np.ones(mesh.num_cells(),dtype='int')*rank
cell_owner = mesh.topology().cell_owner()
if len(cell_owner) > 0 :
    cell_ownership[-len(cell_owner):] = cell_owner

cells_store=[]
cells_note=[]
colors=[]
cmap=['red', 'green', 'yellow', 'purple', 'pink', 'grey', 'blue', 'brown']

idx = 0
for c in cells(mesh, "all"):
    xc=[]
    yc=[]
    for v in vertices(c):
        xc.append(v.point().x())
        yc.append(v.point().y())
    xavg = c.midpoint().x()
    yavg = c.midpoint().y()
    cell_str=str(c.index())
#    if c.index() in shared_cells.keys():
#        cell_str = str(shared_cells[c.index()])
#    else:
#        cell_str = str(c.index())
    cells_note.append((xavg, yavg, cell_str))
    cells_store.append(list(zip(xc,yc)))

    colors.append(cmap[cell_ownership[c.index()]])
    idx += 1

num_regular_facets = mesh.topology().ghost_offset(1)
facet_note = []
shared_facets = mesh.topology().shared_entities(1)
for f in facets(mesh, "all"):
    if (f.num_global_entities(2) == 2):
        color='#ffff88'
    else:
        color='#ff88ff'
    if (not f.is_ghost()):
        if (f.num_global_entities(2) == 2):
            color='#ffff00'
        else:
            color='#ff00ff'

    if (n < 3):
        facet_note.append((f.midpoint().x(), f.midpoint().y(), f.global_index(), color))
    elif (n == 3):
        facet_note.append((f.midpoint().x(), f.midpoint().y(), f.index(), color))
    else:
        if (f.index() in shared_facets.keys()):
            facet_note.append((f.midpoint().x(), f.midpoint().y(), shared_facets[f.index()], color))

fig, ax = plt.subplots()

# Make the collection and add it to the plot.
coll = PolyCollection(cells_store, facecolors=colors, edgecolors='#cccccc')
ax.add_collection(coll)

plt.plot(x, y, marker='o', color='black', linestyle='none')
plt.plot(x[shared_vertices], y[shared_vertices], marker='o', color='green', linestyle='none')
plt.plot(x[ghost_vertices], y[ghost_vertices], marker='o', color='yellow', linestyle='none')

xlim = ax.get_xlim()
ylim = ax.get_ylim()

plt.xlim((xlim[0] - 0.1, xlim[1] + 0.1))
plt.ylim((ylim[0] - 0.1, ylim[1] + 0.1))

for note in cells_note:
    plt.text(note[0], note[1], note[2], verticalalignment='center',
             horizontalalignment='center', size=8)

for note in verts_note:
    plt.text(note[0], note[1], note[2], size=8, verticalalignment='center')

for note in facet_note:
    plt.text(note[0], note[1], note[2], size=8, verticalalignment='center', backgroundcolor=note[3])

# Q = MeshFunction("double", mesh, mesh.topology().dim()-1)

# # Save solution in XDMF format if available
# xdmf = XDMFFile(mesh.mpi_comm(), "Q.xdmf")
# if has_hdf5():
#     xdmf.write(Q)
# elif MPI.size(mesh.mpi_comm()) == 1:
#     encoding = XDMFFile.Encoding.ASCII
#     xdmf.write(Q, encoding)
# else:
#     # Save solution in vtk format
#     xdmf = File("Q.pvd")
#     xdmf << Q

plt.savefig("mesh-rank%d.png" % rank)
if os.environ.get("DOLFIN_NOPLOT", "0") == "0":
    plt.show()



"""This demo program demonstrates how to mark sub domains of a mesh
and store the sub domain markers as a mesh function to a DOLFIN XML
file.

The sub domain markers produced by this demo program are the ones used
for the Stokes demo programs.
"""

# Copyright (C) 2007 Kristian B. Oelgaard
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# Modified by Anders Logg, 2008.
#
# First added:  2007-11-15
# Last changed: 2011-01-25
# Begin demo

from dolfin import *

set_log_level(1)

# Sub domain for no-slip (mark whole boundary, inflow and outflow will overwrite)
class Noslip(SubDomain):
    def inside(self, x, on_boundary):
        return on_boundary

# Sub domain for inflow (right)
class Inflow(SubDomain):
    def inside(self, x, on_boundary):
        return x[0] > 1.0 - DOLFIN_EPS and on_boundary

# Sub domain for outflow (left)
class Outflow(SubDomain):
    def inside(self, x, on_boundary):
        return x[0] < DOLFIN_EPS and on_boundary

# Read mesh
mesh = Mesh("../dolfin_fine.xml.gz")

# Create mesh functions over the cell facets
sub_domains = MeshFunction("size_t", mesh, mesh.topology().dim() - 1)
sub_domains_bool = MeshFunction("bool", mesh, mesh.topology().dim() - 1)
sub_domains_double = MeshFunction("double", mesh, mesh.topology().dim() - 1)

# Mark all facets as sub domain 3
sub_domains.set_all(3)
sub_domains_bool.set_all(False)
sub_domains_double.set_all(0.3)

# Mark no-slip facets as sub domain 0, 0.0
noslip = Noslip()
noslip.mark(sub_domains, 0)
noslip.mark(sub_domains_double, 0.0)

# Mark inflow as sub domain 1, 01
inflow = Inflow()
inflow.mark(sub_domains, 1)
inflow.mark(sub_domains_double, 0.1)

# Mark outflow as sub domain 2, 0.2, True
outflow = Outflow()
outflow.mark(sub_domains, 2)
outflow.mark(sub_domains_double, 0.2)
outflow.mark(sub_domains_bool, True)

# Save sub domains to file
file = File("subdomains.xml")
file << sub_domains

# FIXME: Not implemented
#file_bool = File("subdomains_bool.xml")
#file_bool << sub_domains_bool

file_double = File("subdomains_double.xml")
file_double << sub_domains_double

# Save sub domains to VTK files
file = File("subdomains.pvd")
file << sub_domains

file = File("subdomains_double.pvd")
file << sub_domains_double



# Copyright (C) 2011 Marie E. Rognes
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# First added:  2011-11-09
# Last changed: 2012-08-06

# Begin demo


from dolfin import *

# Create classes for defining parts of the boundaries and the interior
# of the domain
class Left(SubDomain):
    def inside(self, x, on_boundary):
        return near(x[0], 0.0)

class Right(SubDomain):
    def inside(self, x, on_boundary):
        return near(x[0], 1.0)

class Bottom(SubDomain):
    def inside(self, x, on_boundary):
        return near(x[1], 0.0)

class Top(SubDomain):
    def inside(self, x, on_boundary):
        return near(x[1], 1.0)

class Obstacle(SubDomain):
    def inside(self, x, on_boundary):
        return (between(x[1], (0.5, 0.7)) and between(x[0], (0.2, 1.0)))

# Initialize sub-domain instances
left = Left()
top = Top()
right = Right()
bottom = Bottom()
obstacle = Obstacle()

# Define mesh
mesh = UnitSquareMesh(64, 64)

# Initialize mesh function for interior domains
domains = MeshFunction("size_t", mesh, mesh.topology().dim())
domains.set_all(0)
obstacle.mark(domains, 1)

# Initialize mesh function for boundary domains
boundaries = MeshFunction("size_t", mesh, mesh.topology().dim()-1)
boundaries.set_all(0)
left.mark(boundaries, 1)
top.mark(boundaries, 2)
right.mark(boundaries, 3)
bottom.mark(boundaries, 4)

# Define input data
a0 = Constant(1.0)
a1 = Constant(0.01)
g_L = Expression("- 10*exp(- pow(x[1] - 0.5, 2))", degree=2)
g_R = Constant(1.0)
f = Constant(1.0)

# Define function space and basis functions
V = FunctionSpace(mesh, "CG", 2)
u = TrialFunction(V)
v = TestFunction(V)

# Define Dirichlet boundary conditions at top and bottom boundaries
bcs = [DirichletBC(V, 5.0, boundaries, 2),
       DirichletBC(V, 0.0, boundaries, 4)]

# Define new measures associated with the interior domains and
# exterior boundaries
dx = Measure('dx', domain=mesh, subdomain_data=domains)
ds = Measure('ds', domain=mesh, subdomain_data=boundaries)

# Define variational form
F = (inner(a0*grad(u), grad(v))*dx(0) + inner(a1*grad(u), grad(v))*dx(1)
     - g_L*v*ds(1) - g_R*v*ds(3)
     - f*v*dx(0) - f*v*dx(1))

# Separate left and right hand sides of equation
a, L = lhs(F), rhs(F)

# Solve problem
u = Function(V)
solve(a == L, u, bcs)

# Evaluate integral of normal gradient over top boundary
n = FacetNormal(mesh)
m1 = dot(grad(u), n)*ds(2)
v1 = assemble(m1)
print("\int grad(u) * n ds(2) = ", v1)

# Evaluate integral of u over the obstacle
m2 = u*dx(1)
v2 = assemble(m2)
print("\int u dx(1) = ", v2)

# Plot solution
import matplotlib.pyplot as plt
plt.figure()
plot(u, title="Solution u")

# Plot solution and gradient
plt.figure()
plot(grad(u), title="Projected grad(u)")

# Show plots
plt.show()



# Copyright (C) 2011-2013 Marie E. Rognes, Martin S. Alnaes
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# First added:  2011-11-09
# Last changed: 2013-01-29

# Begin demo

#
# NB! This demo is work in progress and will be updated as we get further in the multidomain features implementation.
#
from dolfin import *
import matplotlib.pyplot as plt


class Left(SubDomain):
    def inside(self, x, on_boundary):
        return (x[0] <= 0.25+DOLFIN_EPS)

class Mid(SubDomain):
    def inside(self, x, on_boundary):
        return (x[0] >= 0.375-DOLFIN_EPS) and (x[0] <= 0.625+DOLFIN_EPS)

class Right(SubDomain):
    def inside(self, x, on_boundary):
        return (x[0] >= 0.75-DOLFIN_EPS)

# Define mesh and subdomains
mesh = UnitSquareMesh(64, 64)
domains = MeshFunction("size_t", mesh, mesh.topology().dim())
domains.set_all(0)
Left().mark(domains, 1)
Mid().mark(domains, 2)
Right().mark(domains, 3)

# Define input data
alpha = Constant(1e-3)
f = Constant(3.0)
g = Constant(5.0)

# Define function space and basis functions
V = FunctionSpace(mesh, "CG", 2)
u = TrialFunction(V)
v = TestFunction(V)

# Define regions as tuples of subdomain labels
DL, DM, DR = (1,2), (2,), (2,3) # ***

# Define new measures associated with the interior domains
dx = Measure('dx', domain=mesh, subdomain_data=domains)

# Make forms for equation
a = u*v*dx() + alpha*dot(grad(u), grad(v))*dx() # ***
L = f*v*dx(DR) + g*v*dx(DL) - (f+g)/2*v*dx(DM) # ***

# Solve problem
u = Function(V)
solve(a == L, u)

# Plot solution and gradient
plt.figure()
plot(domains, title='domains')
plt.figure()
plot(u, title="u")
plt.show()



"""
This program demonstrates interpolation onto a mesh which is not
completely covered by the original mesh. This situation may arise
during mesh refinement as a result of round-off errors.

When interpolating, DOLFIN tries to locate each point in the original
mesh. If that fails, the closest cell is found and the values
extrapolated from there.
"""

# Copyright (C) 2010 Anders Logg
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.

from dolfin import *
import matplotlib.pyplot as plt


# Set option to allow extrapolation
parameters["allow_extrapolation"] = True

# Create mesh and function space
mesh = UnitSquareMesh(16, 16)
V = FunctionSpace(mesh, "CG", 1)

# Create a function on the original mesh
f = Expression("sin(5.0*x[0])*sin(5.0*x[1])", degree=2)
v = interpolate(f, V)

# Refine mesh and create a new function space
refined_mesh = refine(mesh)
W = FunctionSpace(refined_mesh, "CG", 1)

# Displace mesh slightly
x = mesh.coordinates()
x += 0.1

# Interpolate to the new mesh
w = interpolate(v, W)

# Plot function
plot(w)
plt.show()



# -*- coding: utf-8 -*-
"""Unit tests for the MultiMeshFunction class"""

# Copyright (C) 2016 Jørgen S. Dokken
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
#
# First added:  2016-06-11
# Last changed: 2017-07-18

import pytest
from dolfin import *
import ufl
import numpy

from dolfin_utils.test import fixture, skip_in_parallel

@fixture
def multimesh():
    mesh_0 = RectangleMesh(Point(0,0), Point(1, 1), 20, 20)
    mesh_1 = RectangleMesh(Point(numpy.pi/10,numpy.pi/9),
                           Point(numpy.pi/8,numpy.pi/7),5,8)
    multimesh = MultiMesh()
    multimesh.add(mesh_0)
    multimesh.add(mesh_1)
    multimesh.build()
    return multimesh

@fixture
def V(multimesh):
    element = FiniteElement("Lagrange", triangle, 1)
    return MultiMeshFunctionSpace(multimesh, element)

@fixture
def V_high(multimesh):
    element = FiniteElement("Lagrange", triangle, 3)
    return MultiMeshFunctionSpace(multimesh, element)

@fixture
def V_vector(multimesh):
    return MultiMeshVectorFunctionSpace(multimesh,"CG", 1)

@fixture
def e_x():
    return Expression(("1.0","0.0"), degree=1)

@fixture
def f_vec2():
    return Expression(("sin(x[1])","1.0"), degree=3)

@fixture
def v_vec(V_vector):
    return MultiMeshFunction(V_vector)

@fixture
def f_vec():
    return Expression(("x[0]","x[1]"),degree=2)


@fixture
def f():
    return Expression("sin(pi*x[0])*cos(2*pi*x[1])", degree=4)

@fixture
def f_2():
    return Expression("x[0]*x[1]",degree=2)
@fixture
def v(V):
    return MultiMeshFunction(V)

@fixture
def v_high(V_high):
    return MultiMeshFunction(V_high)

@skip_in_parallel
def test_measure_mul(v, multimesh):
    assert isinstance(v*dX, ufl.form.Form)

@skip_in_parallel
def test_assemble_zero(v, multimesh):
    assert numpy.isclose(assemble_multimesh(v*dX), 0)

@skip_in_parallel
def test_assemble_area(v, multimesh):
    v.vector()[:] = 1
    assert numpy.isclose(assemble_multimesh(v*dX), 1)

@skip_in_parallel
def test_assemble_exterior_facet(v, multimesh):
    v.vector()[:] = 1
    assert numpy.isclose(assemble_multimesh(v*ds), 4)

@skip_in_parallel
def test_interpolate(v_high, f):
    v_high.interpolate(f)
    assert numpy.isclose(assemble_multimesh(v_high*dX), 0)

@skip_in_parallel
def test_project(f, V_high):
    v = project(f ,V_high)
    assert numpy.isclose(assemble_multimesh(v*dX), 0)

@skip_in_parallel
def test_errornorm_L2(f_2, v_high):
    const = Expression("1", degree=1)
    v_high.interpolate(f_2)
    assert numpy.isclose(errornorm(const, v_high, norm_type="L2", degree_rise=3), numpy.sqrt(22)/6)

@skip_in_parallel
def test_errornorm_H1(f, f_2, v_high):
    v_high.interpolate(f_2)
    assert numpy.isclose(errornorm(f, v_high, norm_type="H1", degree_rise=3), numpy.sqrt(37./36+5*numpy.pi**2/4))

@skip_in_parallel
def test_vector_space(v_vec,e_x,f_vec2,V_vector,f_vec):
    v_vec.interpolate(e_x)
    assert numpy.isclose(assemble_multimesh(inner(f_vec,v_vec)*dX), 0.5)
    v_vec.interpolate(f_vec)
    print((assemble_multimesh(inner(v_vec, f_vec2)*dX),
                                            1/2 + numpy.sin(1/2)*numpy.sin(1/2)))
    assert numpy.isclose(assemble_multimesh(inner(v_vec, f_vec2)*dX),
                                            1/2 + numpy.sin(1/2)*numpy.sin(1/2))



"""Unit tests for graph building"""

# Copyright (C) 2013 Garth N. Wells
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# First added:  2013-08-10
# Last changed:

import pytest
from dolfin import *


def test_build_from_mesh_simple():
    """Build mesh graph """

    mesh = UnitCubeMesh(16, 16, 16)
    D = mesh.topology().dim()
    GraphBuilder.local_graph(mesh, D, 0)
    GraphBuilder.local_graph(mesh, D, 1)
    GraphBuilder.local_graph(mesh, 2, D)
    GraphBuilder.local_graph(mesh, 1, D)
    GraphBuilder.local_graph(mesh, 0, D)



"""Unit tests for the fem interface"""

# Copyright (C) 2009 Garth N. Wells
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# First added:  2009-07-28
# Last changed: 2009-07-28

import pytest
import numpy
from dolfin import *
from dolfin_utils.test import fixture


xfail = pytest.mark.xfail(strict=True)


@pytest.mark.parametrize('mesh_factory', [(UnitSquareMesh, (4, 4)),
                                          (UnitCubeMesh, (2, 2, 2)),
                                          (UnitSquareMesh.create, (4, 4, CellType.Type.quadrilateral)),
                                          # cell_normal has not been implemented for hex cell
                                          # cell.orientation() does not work
                                          pytest.param(((UnitCubeMesh.create, (2, 2, 2, CellType.Type.hexahedron))), marks=xfail)])
def test_evaluate_dofs(mesh_factory):

    func, args = mesh_factory
    mesh = func(*args)

    v = FiniteElement("Lagrange", mesh.ufl_cell(), 1)
    q = VectorElement("Lagrange", mesh.ufl_cell(), 1)
    w = v*q

    V = FunctionSpace(mesh, v)
    W = FunctionSpace(mesh, w)

    sdim = V.element().space_dimension()
    gdim = V.element().geometric_dimension()

    e = Expression("x[0] + x[1]", degree=1)
    e2 = Expression(["x[0] + x[1]"]*gdim, degree=1)

    coords = numpy.zeros((sdim, gdim), dtype="d")
    coord = numpy.zeros(gdim, dtype="d")
    values0 = numpy.zeros(sdim, dtype="d")
    values1 = numpy.zeros(sdim, dtype="d")
    values2 = numpy.zeros(sdim, dtype="d")
    values3 = numpy.zeros(sdim, dtype="d")
    values4 = numpy.zeros(gdim*sdim, dtype="d")

    L0 = W.sub(0)
    L1 = W.sub(1)
    L01 = L1.sub(0)
    L11 = L1.sub(1)

    for cell in cells(mesh):
        vx = cell.get_vertex_coordinates()
        orientation = cell.orientation()
        coords = V.element().tabulate_dof_coordinates(cell)
        for i in range(coords.shape[0]):
            coord[:] = coords[i, :]
            values0[i] = e(*coord)
        values1 = L0.element().evaluate_dofs(e, vx, orientation, cell)
        values2 = L01.element().evaluate_dofs(e, vx, orientation, cell)
        values3 = L11.element().evaluate_dofs(e, vx, orientation, cell)
        values4 = L1.element().evaluate_dofs(e2, vx, orientation, cell)

        for i in range(sdim):
            assert round(values0[i] - values1[i], 7) == 0
            assert round(values0[i] - values2[i], 7) == 0
            assert round(values0[i] - values3[i], 7) == 0
            assert round(values4[:sdim][i] - values0[i], 7) == 0
            if gdim == 3:
                assert round(values4[sdim:sdim*2][i] - values0[i], 7) == 0
                assert round(values4[sdim*2:][i] - values0[i], 7) == 0
            else:
                assert round(values4[sdim:][i] - values0[i], 7) == 0


def test_evaluate_dofs_manifolds_affine():
    "Testing evaluate_dofs vs tabulated coordinates."

    n = 4
    mesh = BoundaryMesh(UnitSquareMesh(n, n), "exterior")
    mesh2 = BoundaryMesh(UnitCubeMesh(n, n, n), "exterior")
    DG0 = FunctionSpace(mesh, "DG", 0)
    DG1 = FunctionSpace(mesh, "DG", 1)
    CG1 = FunctionSpace(mesh, "CG", 1)
    CG2 = FunctionSpace(mesh, "CG", 2)
    DG20 = FunctionSpace(mesh2, "DG", 0)
    DG21 = FunctionSpace(mesh2, "DG", 1)
    CG21 = FunctionSpace(mesh2, "CG", 1)
    CG22 = FunctionSpace(mesh2, "CG", 2)
    elements = [DG0, DG1, CG1, CG2, DG20, DG21, CG21, CG22]

    f = Expression("x[0] + x[1]", degree=1)
    for V in elements:
        sdim = V.element().space_dimension()
        gdim = V.mesh().geometry().dim()
        coord = numpy.zeros(gdim, dtype="d")
        values0 = numpy.zeros(sdim, dtype="d")
        values1 = numpy.zeros(sdim, dtype="d")
        for cell in cells(V.mesh()):
            vx = cell.get_vertex_coordinates()
            orientation = cell.orientation()
            coords = V.element().tabulate_dof_coordinates(cell)
            for i in range(coords.shape[0]):
                coord[:] = coords[i, :]
                values0[i] = f(*coord)
            values1 = V.element().evaluate_dofs(f, vx, orientation, cell)
            for i in range(sdim):
                assert round(values0[i] - values1[i], 7) == 0


@pytest.mark.parametrize('mesh_factory', [(UnitSquareMesh, (4, 4)),
                                          (UnitCubeMesh, (2, 2, 2)),
                                          (UnitSquareMesh.create, (4, 4, CellType.Type.quadrilateral)),
                                          (UnitCubeMesh.create, (2, 2, 2, CellType.Type.hexahedron))])
def test_tabulate_coord(mesh_factory):

    func, args = mesh_factory
    mesh = func(*args)

    v = FiniteElement("Lagrange", mesh.ufl_cell(), 1)
    q = VectorElement("Lagrange", mesh.ufl_cell(), 1)
    w = v*q

    V = FunctionSpace(mesh, v)
    W = FunctionSpace(mesh, w)

    sdim = V.element().space_dimension()
    gdim = V.element().geometric_dimension()
    coord0 = numpy.zeros((sdim, gdim), dtype="d")
    coord1 = numpy.zeros((sdim, gdim), dtype="d")
    coord2 = numpy.zeros((sdim, gdim), dtype="d")
    coord3 = numpy.zeros((sdim, gdim), dtype="d")
    coord4 = numpy.zeros((gdim*sdim, gdim), dtype="d")

    L0 = W.sub(0)
    L1 = W.sub(1)
    L01 = L1.sub(0)
    L11 = L1.sub(1)

    for cell in cells(mesh):
        coord0 = V.element().tabulate_dof_coordinates(cell)
        coord1 = L0.element().tabulate_dof_coordinates(cell)
        coord2 = L01.element().tabulate_dof_coordinates(cell)
        coord3 = L11.element().tabulate_dof_coordinates(cell)
        coord4 = L1.element().tabulate_dof_coordinates(cell)

        assert (coord0 == coord1).all()
        assert (coord0 == coord2).all()
        assert (coord0 == coord3).all()
        if gdim == 3:
            assert (coord4[sdim:sdim*2] == coord0).all()
            assert (coord4[sdim*2:] == coord0).all()
        else:
            assert (coord4[sdim:] == coord0).all()



"This demo illustrates mesh refinement."

# Copyright (C) 2007-2009 Anders Logg
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# First added:  2007-06-01
# Last changed: 2012-11-12


from dolfin import *
import matplotlib.pyplot as plt


# Create mesh of unit square
mesh = UnitSquareMesh(8, 8)
plt.figure(1)
plot(mesh)

info(mesh)
print()

# Uniform refinement
mesh = refine(mesh)
plt.figure(2)
plot(mesh)

info(mesh)
print()

# Uniform refinement
mesh = refine(mesh)
plt.figure(3)
plot(mesh)

info(mesh)
print()

# Refine mesh close to x = (0.5, 0.5)
p = Point(0.5, 0.5)
for i in range(5):

    print("marking for refinement")

    # Mark cells for refinement
    cell_markers = MeshFunction("bool", mesh, mesh.topology().dim())
    for c in cells(mesh):
        if c.midpoint().distance(p) < 0.1:
            cell_markers[c] = True
        else:
            cell_markers[c] = False

    # Refine mesh
    mesh = refine(mesh, cell_markers)

    # Plot mesh
    plt.figure()
    plot(mesh)

plt.show()



# -*- coding: utf-8 -*-
"""This module defines different MultiStageScheme classes which can be
passed to a RKSolver or PointIntegralSolver

"""

# Copyright (C) 2013-2015 Johan Hake
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# Modified by Patrick Farrell, 2013
# Modified by Martin Sandve Alnæs, 2015

import numpy as np
import functools
import ufl

import dolfin.cpp as cpp
from dolfin.function.constant import Constant
from dolfin.function.expression import Expression
from dolfin.function.function import Function
from dolfin.fem.formmanipulations import derivative, adjoint
from ufl import action as ufl_action
from dolfin.fem.form import Form
import ufl.algorithms
from ufl.algorithms import expand_derivatives

# FIXME: Add support for algebraic parts (at least for implicit)
# FIXME: Add support for implicit/explicit split ala IMEX schemes


def safe_adjoint(x):
    return adjoint(x, reordered_arguments=x.arguments())


def safe_action(x, y):
    x = expand_derivatives(x)
    if x.integrals() == ():
        return x  # form is empty, return anyway
    else:
        return ufl_action(x, y)


def _check_abc(a, b, c):
    if not (isinstance(a, np.ndarray) and (len(a) == 1 or
            (len(a.shape) == 2 and a.shape[0] == a.shape[1]))):
        raise TypeError("Expected an m x m numpy array as the first argument")
    if not (isinstance(b, np.ndarray) and len(b.shape) in [1, 2]):
        raise TypeError("Expected a 1 or 2 dimensional numpy array as the second argument")
    if not (isinstance(c, np.ndarray) and len(c.shape) == 1):
        raise TypeError("Expected a 1 dimensional numpy array as the third argument")

    # Make sure a is a "matrix"
    if len(a) == 1:
        a.shape = (1, 1)

    # Get size of system
    size = a.shape[0]

    # If b is a matrix we expect it to have two rows
    if len(b.shape) == 2:
        if not (b.shape[0] == 2 and b.shape[1] == size):
            raise ValueError("Expected a 2 row matrix with the same number "
                             "of collumns as the first dimension of the a matrix.")
    elif len(b) != size:
        raise ValueError("Expected the length of the b vector to have the "
                         "same size as the first dimension of the a matrix.")

    if len(c) != size:
        raise ValueError("Expected the length of the c vector to have the "
                         "same size as the first dimension of the a matrix.")

    # Check if the method is singly diagonally implicit
    sigma = -1
    for i in range(size):
        # If implicit
        if a[i, i] != 0:
            if sigma == -1:
                sigma = a[i, i]
            elif sigma != a[i, i]:
                raise ValueError("Expected only singly diagonally implicit "
                                 "schemes. (Same value on the diagonal of 'a'.)")

    # Check if tableau is fully implicit
    for i in range(size):
        for j in range(i):
            if a[j, i] != 0:
                raise ValueError("Does not support fully implicit Butcher tableau.")

    return a


def _check_form(rhs_form):
    if not isinstance(rhs_form, ufl.Form):
        raise TypeError("Expected a ufl.Form as the 5th argument.")

    # Check if form contains a cell or point integral
    if rhs_form.integrals_by_type("cell"):
        DX = ufl.dx
    elif rhs_form.integrals_by_type("vertex"):
        DX = ufl.dP
    else:
        raise ValueError("Expected either a cell or vertex integral in the form.")

    if len(rhs_form.integrals()) != 1:
        raise ValueError("Expected only one integral in form.")

    arguments = rhs_form.arguments()
    if len(arguments) != 1:
        raise ValueError("Expected the form to have rank 1")

    return DX


def _time_dependent_expressions(rhs_form, time):
    """Return a list of expressions which uses the present time as a
    parameter

    """
    # FIXME: Add extraction of time dependant expressions from bcs too
    time_dependent_expressions = dict()

    for coefficient in rhs_form.coefficients():
        if hasattr(coefficient, "_user_parameters"):
            for c_name, c in list(coefficient._user_parameters.items()):
                if isinstance(c, ufl.Coefficient) and time.id() == c.id():
                    if coefficient not in time_dependent_expressions:
                        time_dependent_expressions[coefficient] = [c_name]
                    else:
                        time_dependent_expressions[coefficient].append(c_name)

    return time_dependent_expressions


def _replace_dict_time_dependent_expression(time_dep_expressions, time,
                                            dt, c):
    assert (isinstance(c, float))
    replace_dict = {}
    if c == 0.0 or not time_dep_expressions:
        return replace_dict
    new_time = Expression("time + c*dt", time=time, c=c, dt=dt, degree=0)
    for expr, c_names in list(time_dep_expressions.items()):
        assert (isinstance(expr, Expression))
        kwargs = dict(name=expr.name(), label=expr.label(),
                      element=expr.ufl_element(), **expr._user_parameters)
        for c_name in c_names:
            kwargs[c_name] = new_time
        replace_dict[expr] = Expression(expr._cppcode, **kwargs)

    return replace_dict


def _butcher_scheme_generator(a, b, c, time, solution, rhs_form):
    """Generates a list of forms and solutions for a given Butcher tableau

    *Arguments*
        a (2 dimensional numpy array)
            The a matrix of the Butcher tableau.
        b (1-2 dimensional numpy array)
            The b vector of the Butcher tableau. If b is 2 dimensional the
            scheme includes an error estimator and can be used in adaptive
            solvers.
        c (1 dimensional numpy array)
            The c vector the Butcher tableau.
        time (_Constant_)
            A Constant holding the time at the start of the time step
        solution (_Function_)
            The prognostic variable
        rhs_form (ufl.Form)
            A UFL form representing the rhs for a time differentiated equation

    """

    a = _check_abc(a, b, c)
    size = a.shape[0]

    DX = _check_form(rhs_form)

    # Get test function
    arguments = rhs_form.arguments()
    # coefficients = rhs_form.coefficients()
    v = arguments[0]

    # Create time step
    dt = Constant(0.1)

    # rhs forms
    dolfin_stage_forms = []
    ufl_stage_forms = []

    # Stage solutions
    k = [Function(solution.function_space(), name="k_%d" % i) for i in range(size)]

    jacobian_indices = []

    # Create the stage forms
    y_ = solution
    time_ = time
    time_dep_expressions = _time_dependent_expressions(rhs_form, time)
    zero_ = ufl.zero(*y_.ufl_shape)
    for i, ki in enumerate(k):

        # Check whether the stage is explicit
        explicit = a[i, i] == 0

        # Evaluation arguments for the ith stage
        evalargs = y_ + dt * sum([float(a[i, j]) * k[j]
                                  for j in range(i + 1)], zero_)
        time = time_ + dt * c[i]

        replace_dict = _replace_dict_time_dependent_expression(time_dep_expressions,
                                                               time_, dt, c[i])

        replace_dict[y_] = evalargs
        replace_dict[time_] = time
        stage_form = ufl.replace(rhs_form, replace_dict)

        if explicit:
            stage_forms = [stage_form]
            jacobian_indices.append(-1)
        else:
            # Create a F=0 form and differentiate it
            stage_form -= ufl.inner(ki, v) * DX
            stage_forms = [stage_form, derivative(stage_form, ki)]
            jacobian_indices.append(0)
        ufl_stage_forms.append(stage_forms)

        dolfin_stage_forms.append([Form(form) for form in stage_forms])

    # Only one last stage
    if len(b.shape) == 1:
        last_stage = Form(ufl.inner(y_ + sum([dt * float(bi) * ki for bi, ki in
                                              zip(b, k)], zero_), v) * DX)
    else:
        # FIXME: Add support for adaptivity in RKSolver and
        # MultiStageScheme
        last_stage = [Form(ufl.inner(y_ + sum([dt * float(bi) * ki for bi, ki in
                                               zip(b[0, :], k)], zero_), v) * DX),
                      Form(ufl.inner(y_ + sum([dt * float(bi) * ki for bi, ki in
                                               zip(b[1, :], k)], zero_), v) * DX)]

    # Create the Function holding the solution at end of time step
    # k.append(solution.copy())

    # Generate human form of MultiStageScheme
    human_form = []
    for i in range(size):
        kterm = " + ".join("%sh*k_%s" % ("" if a[i, j] == 1.0 else
                                         "%s*" % a[i, j], j)
                           for j in range(size) if a[i, j] != 0)
        if c[i] in [0.0, 1.0]:
            cih = " + h" if c[i] == 1.0 else ""
        else:
            cih = " + %s*h" % c[i]

        if len(kterm) == 0:
            human_form.append("k_%(i)s = f(t_n%(cih)s, y_n)" % {"i": i, "cih": cih})
        else:
            human_form.append("k_%(i)s = f(t_n%(cih)s, y_n + %(kterm)s)" %
                              {"i": i, "cih": cih, "kterm": kterm})

    parentheses = "(%s)" if np.sum(b > 0) > 1 else "%s"
    human_form.append("y_{n+1} = y_n + h*" + parentheses % (" + ".join(
        "%sk_%s" % ("" if b[i] == 1.0 else "%s*" % b[i], i)
        for i in range(size) if b[i] > 0)))

    human_form = "\n".join(human_form)

    return ufl_stage_forms, dolfin_stage_forms, jacobian_indices, last_stage, \
        k, dt, human_form, None


def _butcher_scheme_generator_tlm(a, b, c, time, solution, rhs_form,
                                  perturbation):
    """Generates a list of forms and solutions for a given Butcher tableau

    *Arguments*
        a (2 dimensional numpy array)
            The a matrix of the Butcher tableau.
        b (1-2 dimensional numpy array)
            The b vector of the Butcher tableau. If b is 2 dimensional the
            scheme includes an error estimator and can be used in adaptive
            solvers.
        c (1 dimensional numpy array)
            The c vector the Butcher tableau.
        time (_Constant_)
            A Constant holding the time at the start of the time step
        solution (_Function_)
            The prognostic variable
        rhs_form (ufl.Form)
            A UFL form representing the rhs for a time differentiated equation
        perturbation (_Function_)
            The perturbation in the initial condition of the solution

    """

    a = _check_abc(a, b, c)
    size = a.shape[0]

    DX = _check_form(rhs_form)

    # Get test function
    arguments = rhs_form.arguments()
    # coefficients = rhs_form.coefficients()
    v = arguments[0]

    # Create time step
    dt = Constant(0.1)

    # rhs forms
    dolfin_stage_forms = []
    ufl_stage_forms = []

    # Stage solutions
    k = [Function(solution.function_space(), name="k_%d" % i) for i in range(size)]
    kdot = [Function(solution.function_space(), name="kdot_%d" % i)
            for i in range(size)]

    # Create the stage forms
    y_ = solution
    time_ = time
    time_dep_expressions = _time_dependent_expressions(rhs_form, time)
    zero_ = ufl.zero(*y_.ufl_shape)
    forward_forms = []
    stage_solutions = []
    jacobian_indices = []

    for i, ki in enumerate(k):

        # Check whether the stage is explicit
        explicit = a[i, i] == 0

        # Evaluation arguments for the ith stage
        evalargs = y_ + dt * sum([float(a[i, j]) * k[j]
                                  for j in range(i + 1)], zero_)
        time = time_ + dt * c[i]

        replace_dict = _replace_dict_time_dependent_expression(time_dep_expressions,
                                                               time_, dt, c[i])

        replace_dict[y_] = evalargs
        replace_dict[time_] = time
        stage_form = ufl.replace(rhs_form, replace_dict)

        forward_forms.append(stage_form)

        # The recomputation of the forward run:

        if explicit:
            stage_forms = [stage_form]
            jacobian_indices.append(-1)
        else:
            # Create a F=0 form and differentiate it
            stage_form_implicit = stage_form - ufl.inner(ki, v) * DX
            stage_forms = [stage_form_implicit, derivative(stage_form_implicit, ki)]
            jacobian_indices.append(0)

        ufl_stage_forms.append(stage_forms)
        dolfin_stage_forms.append([Form(form) for form in stage_forms])
        stage_solutions.append(ki)

        # And now the tangent linearisation:
        stage_form_tlm = safe_action(derivative(stage_form, y_), perturbation) + \
            sum([dt * float(a[i, j]) * safe_action(derivative(
                forward_forms[j], y_), kdot[j]) for j in range(i + 1)])
        if explicit:
            stage_forms_tlm = [stage_form_tlm]
            jacobian_indices.append(-1)
        else:
            # Create a F=0 form and differentiate it
            stage_form_tlm -= ufl.inner(kdot[i], v) * DX
            stage_forms_tlm = [stage_form_tlm, derivative(stage_form_tlm, kdot[i])]
            jacobian_indices.append(1)

        ufl_stage_forms.append(stage_forms_tlm)
        dolfin_stage_forms.append([Form(form) for form in stage_forms_tlm])
        stage_solutions.append(kdot[i])

    # Only one last stage
    if len(b.shape) == 1:
        last_stage = Form(ufl.inner(perturbation + sum(
            [dt * float(bi) * kdoti for bi, kdoti in zip(b, kdot)], zero_), v) * DX)
    else:
        raise Exception("Not sure what to do here")

    human_form = []
    for i in range(size):
        kterm = " + ".join("%sh*k_%s" % ("" if a[i, j] == 1.0 else
                                         "%s*" % a[i, j], j)
                           for j in range(size) if a[i, j] != 0)
        if c[i] in [0.0, 1.0]:
            cih = " + h" if c[i] == 1.0 else ""
        else:
            cih = " + %s*h" % c[i]

        kdotterm = " + ".join("%(a)sh*action(derivative(f(t_n%(cih)s, y_n + "
                              "%(kterm)s), kdot_%(i)s" %
                              {"a": ("" if a[i, j] == 1.0 else "%s*" % a[i, j], j),
                               "i": i,
                               "cih": cih,
                               "kterm": kterm}
                              for j in range(size) if a[i, j] != 0)

        if len(kterm) == 0:
            human_form.append("k_%(i)s = f(t_n%(cih)s, y_n)" % {"i": i, "cih": cih})
            human_form.append("kdot_%(i)s = action(derivative("
                              "f(t_n%(cih)s, y_n), y_n), ydot_n)" %
                              {"i": i, "cih": cih})
        else:
            human_form.append("k_%(i)s = f(t_n%(cih)s, y_n + %(kterm)s)" %
                              {"i": i, "cih": cih, "kterm": kterm})
            human_form.append("kdot_%(i)s = action(derivative(f(t_n%(cih)s, "
                              "y_n + %(kterm)s), y_n) + %(kdotterm)s" %
                              {"i": i, "cih": cih, "kterm": kterm, "kdotterm": kdotterm})

    parentheses = "(%s)" if np.sum(b > 0) > 1 else "%s"
    human_form.append("ydot_{n+1} = ydot_n + h*" + parentheses % (" + ".join(
        "%skdot_%s" % ("" if b[i] == 1.0 else "%s*" % b[i], i)
        for i in range(size) if b[i] > 0)))

    human_form = "\n".join(human_form)

    return ufl_stage_forms, dolfin_stage_forms, jacobian_indices, last_stage, \
        stage_solutions, dt, human_form, perturbation


def _butcher_scheme_generator_adm(a, b, c, time, solution, rhs_form, adj):
    """Generates a list of forms and solutions for a given Butcher tableau

    *Arguments*
        a (2 dimensional numpy array)
            The a matrix of the Butcher tableau.
        b (1-2 dimensional numpy array)
            The b vector of the Butcher tableau. If b is 2 dimensional the
            scheme includes an error estimator and can be used in adaptive
            solvers.
        c (1 dimensional numpy array)
            The c vector the Butcher tableau.
        time (_Constant_)
            A Constant holding the time at the start of the time step
        solution (_Function_)
            The prognostic variable
        rhs_form (ufl.Form)
            A UFL form representing the rhs for a time differentiated equation
        adj (_Function_)
            The derivative of the functional with respect to y_n+1

    """

    a = _check_abc(a, b, c)
    size = a.shape[0]

    DX = _check_form(rhs_form)

    # Get test function
    arguments = rhs_form.arguments()
    # coefficients = rhs_form.coefficients()
    v = arguments[0]

    # Create time step
    dt = Constant(0.1)

    # rhs forms
    dolfin_stage_forms = []
    ufl_stage_forms = []

    # Stage solutions
    k = [Function(solution.function_space(), name="k_%d" % i) for i in range(size)]
    kbar = [Function(solution.function_space(), name="kbar_%d" % i)
            for i in range(size)]

    # Create the stage forms
    y_ = solution
    time_ = time
    time_dep_expressions = _time_dependent_expressions(rhs_form, time)
    zero_ = ufl.zero(*y_.ufl_shape)
    forward_forms = []
    stage_solutions = []
    jacobian_indices = []

    # The recomputation of the forward run:
    for i, ki in enumerate(k):

        # Check whether the stage is explicit
        explicit = a[i, i] == 0

        # Evaluation arguments for the ith stage
        evalargs = y_ + dt * sum([float(a[i, j]) * k[j]
                                  for j in range(i + 1)], zero_)
        time = time_ + dt * c[i]

        replace_dict = _replace_dict_time_dependent_expression(
            time_dep_expressions, time_, dt, c[i])

        replace_dict[y_] = evalargs
        replace_dict[time_] = time
        stage_form = ufl.replace(rhs_form, replace_dict)

        forward_forms.append(stage_form)

        if explicit:
            stage_forms = [stage_form]
            jacobian_indices.append(-1)
        else:
            # Create a F=0 form and differentiate it
            stage_form_implicit = stage_form - ufl.inner(ki, v) * DX
            stage_forms = [stage_form_implicit, derivative(
                stage_form_implicit, ki)]
            jacobian_indices.append(0)

        ufl_stage_forms.append(stage_forms)
        dolfin_stage_forms.append([Form(form) for form in stage_forms])
        stage_solutions.append(ki)

    for i, kbari in reversed(list(enumerate(kbar))):

        # Check whether the stage is explicit
        explicit = a[i, i] == 0

        # And now the adjoint linearisation:
        stage_form_adm = ufl.inner(dt * b[i] * adj, v) * DX + sum(
            [dt * float(a[j, i]) * safe_action(safe_adjoint(derivative(
                forward_forms[j], y_)), kbar[j]) for j in range(i, size)])
        if explicit:
            stage_forms_adm = [stage_form_adm]
            jacobian_indices.append(-1)
        else:
            # Create a F=0 form and differentiate it
            stage_form_adm -= ufl.inner(kbar[i], v) * DX
            stage_forms_adm = [stage_form_adm, derivative(stage_form_adm, kbari)]
            jacobian_indices.append(1)

        ufl_stage_forms.append(stage_forms_adm)
        dolfin_stage_forms.append([Form(form) for form in stage_forms_adm])
        stage_solutions.append(kbari)

    # Only one last stage
    if len(b.shape) == 1:
        last_stage = Form(ufl.inner(adj, v) * DX + sum(
            [safe_action(safe_adjoint(derivative(forward_forms[i], y_)), kbar[i])
             for i in range(size)]))
    else:
        raise Exception("Not sure what to do here")

    human_form = "unimplemented"

    return ufl_stage_forms, dolfin_stage_forms, jacobian_indices, last_stage, \
        stage_solutions, dt, human_form, adj


class MultiStageScheme(cpp.multistage.MultiStageScheme):
    """Base class for all MultiStageSchemes

    """
    def __init__(self, rhs_form, ufl_stage_forms,
                 dolfin_stage_forms, last_stage, stage_solutions,
                 solution, time, dt, dt_stage_offsets, jacobian_indices, order,
                 name, human_form, bcs, contraction=None):

        # Store Python data
        self._rhs_form = rhs_form
        self._ufl_stage_forms = ufl_stage_forms
        self._dolfin_stage_forms = dolfin_stage_forms
        self._t = time
        self._bcs = bcs
        self._dt = dt
        self._last_stage = last_stage
        self._solution = solution
        self._stage_solutions = stage_solutions
        self._order = order
        self.jacobian_indices = jacobian_indices
        self.contraction = contraction

        # Pass args to C++ constructor
        stage_solutions = [s.cpp_object() for s in stage_solutions]

        cpp.multistage.MultiStageScheme.__init__(self,
                                                 dolfin_stage_forms,
                                                 last_stage,
                                                 stage_solutions,
                                                 solution.cpp_object(),
                                                 time.cpp_object(),
                                                 dt.cpp_object(),
                                                 dt_stage_offsets,
                                                 jacobian_indices,
                                                 order,
                                                 self.__class__.__name__,
                                                 human_form, bcs)

    def rhs_form(self):
        "Return the original rhs form"
        return self._rhs_form

    def ufl_stage_forms(self):
        "Return the ufl stage forms"
        return self._ufl_stage_forms

    def dolfin_stage_forms(self):
        "Return the dolfin stage forms"
        return self._dolfin_stage_forms

    def t(self):
        "Return the Constant used to describe time in the MultiStageScheme"
        return self._t

    def dt(self):
        "Return the Constant used to describe time in the MultiStageScheme"
        return self._dt

    def solution(self):
        "Return the solution Function"
        return self._solution

    def last_stage(self):
        "Return the form describing the last stage"
        return self._last_stage

    def stage_solutions(self):
        "Return the stage solutions"
        return self._stage_solutions

    def to_tlm(self, perturbation):
        raise NotImplementedError("'to_tlm:' implement in derived classes")

    def to_adm(self, perturbation):
        raise NotImplementedError("'to_adm:' implement in derived classes")


class ButcherMultiStageScheme(MultiStageScheme):
    """Base class for all MultiStageSchemes

    """
    def __init__(self, rhs_form, solution, time, bcs, a, b, c, order,
                 generator=_butcher_scheme_generator):
        bcs = bcs or []
        time = time or Constant(0.0)
        ufl_stage_forms, dolfin_stage_forms, jacobian_indices, last_stage, \
            stage_solutions, dt, human_form, contraction = \
            generator(a, b, c, time, solution, rhs_form)

        # Store data
        self.a = a
        self.b = b
        self.c = c

        MultiStageScheme.__init__(self, rhs_form, ufl_stage_forms,
                                  dolfin_stage_forms, last_stage,
                                  stage_solutions, solution, time, dt,
                                  c, jacobian_indices, order,
                                  self.__class__.__name__, human_form,
                                  bcs, contraction)

    def to_tlm(self, perturbation):
        r"""Return another MultiStageScheme that implements the tangent
        linearisation of the ODE solver.

        This takes \dot{y_n} (the derivative of y_n with respect to a
        parameter) and computes \dot{y_n+1} (the derivative of y_n+1
        with respect to that parameter).

        """

        generator = functools.partial(_butcher_scheme_generator_tlm,
                                      perturbation=perturbation)
        new_solution = self._solution.copy()
        new_form = ufl.replace(self._rhs_form, {self._solution: new_solution})
        return ButcherMultiStageScheme(new_form, new_solution,
                                       self._t, self._bcs, self.a,
                                       self.b, self.c, self._order,
                                       generator=generator)

    def to_adm(self, adj):
        r"""Return another MultiStageScheme that implements the adjoint
        linearisation of the ODE solver.

        This takes \bar{y_n+1} (the derivative of a functional J with
        respect to y_n+1) and computes \bar{y_n} (the derivative of J
        with respect to y_n).

        """

        generator = functools.partial(_butcher_scheme_generator_adm, adj=adj)
        new_solution = self._solution.copy()
        new_form = ufl.replace(self._rhs_form, {self._solution: new_solution})
        return ButcherMultiStageScheme(new_form, new_solution, self._t, self._bcs,
                                       self.a, self.b, self.c, self._order,
                                       generator=generator)


class ERK1(ButcherMultiStageScheme):
    """Explicit first order Scheme"""
    def __init__(self, rhs_form, solution, t=None, bcs=None):
        a = np.array([0.])
        b = np.array([1.])
        c = np.array([0.])
        ButcherMultiStageScheme.__init__(self, rhs_form, solution, t, bcs,
                                         a, b, c, 1)


class BDF1(ButcherMultiStageScheme):
    """Implicit first order scheme"""
    def __init__(self, rhs_form, solution, t=None, bcs=None):
        a = np.array([1.])
        b = np.array([1.])
        c = np.array([1.])
        ButcherMultiStageScheme.__init__(self, rhs_form, solution, t, bcs,
                                         a, b, c, 1)


class ExplicitMidPoint(ButcherMultiStageScheme):
    """Explicit 2nd order scheme"""
    def __init__(self, rhs_form, solution, t=None, bcs=None):

        a = np.array([[0, 0], [0.5, 0.0]])
        b = np.array([0., 1])
        c = np.array([0, 0.5])
        ButcherMultiStageScheme.__init__(self, rhs_form, solution, t, bcs,
                                         a, b, c, 2)


class CN2(ButcherMultiStageScheme):
    """Semi-implicit 2nd order scheme"""
    def __init__(self, rhs_form, solution, t=None, bcs=None):
        a = np.array([[0, 0], [0.5, 0.5]])
        b = np.array([0.5, 0.5])
        c = np.array([0, 1.0])

        ButcherMultiStageScheme.__init__(self, rhs_form, solution, t, bcs,
                                         a, b, c, 2)


class ERK4(ButcherMultiStageScheme):
    """Explicit 4th order scheme"""
    def __init__(self, rhs_form, solution, t=None, bcs=None):
        a = np.array([[0, 0, 0, 0],
                      [0.5, 0, 0, 0],
                      [0, 0.5, 0, 0],
                      [0, 0, 1, 0]])
        b = np.array([1. / 6, 1. / 3, 1. / 3, 1. / 6])
        c = np.array([0, 0.5, 0.5, 1])
        ButcherMultiStageScheme.__init__(self, rhs_form, solution, t, bcs,
                                         a, b, c, 4)


class ESDIRK3(ButcherMultiStageScheme):
    """Explicit implicit 3rd order scheme

    See also "Singly diagonally implicit Runge–Kutta methods with an
    explicit first stage" by A Kværnø - BIT Numerical Mathematics,
    2004 (p.497)

    """
    def __init__(self, rhs_form, solution, t=None, bcs=None):
        a = np.array([[0.000000000000000, 0.000000000000000, 0.000000000000000, 0.00000000000000],
                      [0.435866521500000, 0.435866521500000, 0.000000000000000, 0.00000000000000],
                      [0.490563388419108, 0.073570090080892, 0.435866521500000, 0.00000000000000],
                      [0.308809969973036, 1.490563388254108, -1.235239879727145, 0.435866521500000]])
        b = a[-1, :].copy()
        c = a.sum(1)
        ButcherMultiStageScheme.__init__(self, rhs_form, solution, t, bcs, a, b, c, 3)


class ESDIRK4(ButcherMultiStageScheme):
    """Explicit implicit 4rd order scheme

    See also "Singly diagonally implicit Runge–Kutta methods with an
    explicit first stage" by A Kværnø - BIT Numerical Mathematics,
    2004 (p.498)

    """
    def __init__(self, rhs_form, solution, t=None, bcs=None):
        a = np.array([[0.000000000000000, 0.0000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000],
                      [0.435866521500000, 0.4358665215000000, 0.000000000000000, 0.000000000000000, 0.000000000000000],
                      [0.140737774731968, -0.108365551378832, 0.435866521500000, 0.000000000000000, 0.000000000000000],
                      [0.102399400616089, -0.376878452267324, 0.838612530151233, 0.435866521500000, 0.000000000000000],
                      [0.157024897860995, 0.1173304413577680, 0.616678030391680, -0.326899891110444, 0.435866521500000]])

        b = a[-1, :].copy()
        c = a.sum(1)
        ButcherMultiStageScheme.__init__(self, rhs_form, solution, t, bcs, a, b, c, 4)


# Aliases
CrankNicolson = CN2
ExplicitEuler = ERK1
ForwardEuler = ERK1
ImplicitEuler = BDF1
BackwardEuler = BDF1
ERK = ERK1
RK4 = ERK4

__all__ = [name for name, attr in list(globals().items())
           if isinstance(attr, type) and issubclass(attr, MultiStageScheme)]

__all__.append("MultiStageScheme")



"""Unit test for the SNES nonlinear solver"""

# Copyright (C) 2012 Patrick E. Farrell
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# First added:  2012-10-17
# Last changed: 2016-10-26

"""Solve the Yamabe PDE which arises in the differential geometry of
general relativity. http://arxiv.org/abs/1107.0360.

The Yamabe equation is highly nonlinear and supports many
solutions. However, only one of these is of physical relevance -- the
positive solution.

This unit test demonstrates the capability of the SNES solver to
accept bounds on the resulting solution. The plain Newton method
converges to an unphysical negative solution, while the SNES solution
with {sign: nonnegative} converges to the physical positive solution.

An alternative interface to SNESVI allows the user to set explicitly
more complex bounds as GenericVectors or Function.
"""

from dolfin import *
import pytest
import os
import numbers

from dolfin_utils.test import *

parameter_degree = set_parameters_fixture("form_compiler.quadrature_degree", \
                                          [5])
parameter_backend = set_parameters_fixture("linear_algebra_backend", ["PETSc"])

@fixture
def mesh(datadir):
    return Mesh(os.path.join(datadir, "doughnut.xml"))

@fixture
def V(mesh):
    return FunctionSpace(mesh, "CG", 1)

@fixture
def bcs(V):
    return [DirichletBC(V, 1.0, "on_boundary")]

@fixture
def u(V):
    u = Function(V)
    u.interpolate(Constant(-1000.0))
    return u

@fixture
def v(V):
    return TestFunction(V)

@fixture
def F(u, v, mesh):
    x = SpatialCoordinate(mesh)
    r = sqrt(x[0]**2 + x[1]**2)
    rho = 1.0/r**3

    return (8*inner(grad(u), grad(v))*dx + rho * inner(u**5, v)*dx \
            + (-1.0/8.0)*inner(u, v)*dx)

@fixture
def J(V, u, F):
    du = TrialFunction(V)
    return derivative(F, u, du)

@fixture
def lb(V):
    return interpolate(Constant(0.), V)

@fixture
def ub(V):
    return interpolate(Constant(100.), V)

@fixture
def newton_solver_parameters():
    return{"nonlinear_solver": "newton",
           "newton_solver": {"linear_solver": "lu",
                             "maximum_iterations": 100,
                             "report": False}}

@fixture
def snes_solver_parameters_sign():
    return {"nonlinear_solver": "snes",
            "snes_solver": {"linear_solver": "lu",
                            "maximum_iterations": 100,
                            "sign": "nonnegative",
                            "report": True}}

@fixture
def snes_solver_parameters_bounds():
    return {"nonlinear_solver": "snes",
            "snes_solver": {"linear_solver": "lu",
                            "maximum_iterations": 100,
                            "sign": "default",
                            "report": True}}


@skip_if_not_PETSc
def test_snes_solver(F, bcs, u, snes_solver_parameters_sign, parameter_degree,\
                     parameter_backend):
    u.interpolate(Constant(-1000.0))
    solve(F == 0, u, bcs, solver_parameters=snes_solver_parameters_sign)
    assert u.vector().min() >= 0


@skip_if_not_PETSc
def test_newton_solver(F, u, bcs, newton_solver_parameters, parameter_degree,\
                       parameter_backend):
    u.interpolate(Constant(-1000.0))
    solve(F == 0, u, bcs, solver_parameters=newton_solver_parameters)
    assert u.vector().min() < 0


@skip_if_not_PETSc
@pytest.mark.skipif(not has_krylov_solver_preconditioner('amg'),
                    reason="This test requires amg.")
def test_preconditioner_interface(V, parameter_backend):
    "Test nonlinear solvers preconditioner interface"
    class Problem(NonlinearProblem):
        def __init__(self, V):
            NonlinearProblem.__init__(self)

            u = Function(V)
            u_ = TrialFunction(V)
            v = TestFunction(V)
            L = Constant(1.0)*v*dx
            bc = DirichletBC(V, 0, lambda x, b: b)

            # Nonlinear problem and its Jacobian
            F = (1.0+u*u)*inner(grad(u), grad(v))*dx - L
            J = derivative(F, u)

            # Preconditioner form
            J_pc = inner(grad(u_), grad(v))*dx

            assembler = SystemAssembler(J, F, bc)
            assembler_pc = SystemAssembler(J_pc, F, bc)

            self.u = u
            self.assembler = assembler
            self.assembler_pc = assembler_pc

        def F(self, b, x):
            self.assembler.assemble(b, x)

        def J(self, A, x):
            self.assembler.assemble(A)

        def J_pc(self, P, x):
            # Assemble preconditioner only once
            if P.empty():
                self.assembler_pc.assemble(P)
                del self.assembler_pc

    class MyNewtonSolver(NewtonSolver):
        def converged(self, r, p, i):
            self._converged_called = True
            assert isinstance(r, GenericVector)
            assert isinstance(p, NonlinearProblem)
            assert isinstance(i, numbers.Integral)
            return super(MyNewtonSolver, self).converged(r, p, i)
        def solver_setup(self, A, J, p, i):
            self._solver_setup_called = True
            assert isinstance(A, cpp.la.GenericMatrix)
            assert isinstance(J, cpp.la.GenericMatrix)
            assert isinstance(p, NonlinearProblem)
            assert isinstance(i, numbers.Integral)
            super(MyNewtonSolver, self).solver_setup(A, J, p, i)
        def update_solution(self, x, dx, rp, p, i):
            self._update_solution_called = True
            assert isinstance(x, cpp.la.GenericVector)
            assert isinstance(dx, cpp.la.GenericVector)
            assert isinstance(rp, float)
            assert isinstance(p, NonlinearProblem)
            assert isinstance(i, numbers.Integral)
            super(MyNewtonSolver, self).update_solution(x, dx, rp, p, i)
        @property
        def check_overloads_called(self):
            assert getattr(self, "_converged_called", False)
            assert getattr(self, "_solver_setup_called", False)
            assert getattr(self, "_update_solution_called", False)

    class MyPETScNewtonSolver(MyNewtonSolver):
        def __init__(self):
            self.petsc_solver = PETScKrylovSolver()
            NewtonSolver.__init__(self, V.mesh().mpi_comm(),
                self.petsc_solver, PETScFactory.instance())
        def update_solution(self, x, dx, rp, p, i):
            self._update_solution_called = True
            assert self.linear_solver() is self.petsc_solver
            super().update_solution(x, dx, rp, p, i)


    for solverclass in [NewtonSolver, MyNewtonSolver, MyPETScNewtonSolver, PETScSNESSolver]:
        problem = Problem(V)
        x = problem.u.vector()

        solver = solverclass()
        solver.parameters["linear_solver"] = "cg"
        solver.parameters["preconditioner"] = "amg"
        solver.parameters["krylov_solver"]["monitor_convergence"] = True
        solver.parameters["report"] = True

        # Check that subsequent solutions work and reuse preconditioner
        solver.solve(problem, x)
        x.zero()
        solver.solve(problem, x)

        # Check that overloading NewtonSolver members works
        getattr(solver, "check_overloads_called", None)


@skip_if_not_PETSc
def test_snes_solver_bound_vectors(F, u, bcs, J,
                                   snes_solver_parameters_bounds,
                                   lb, ub, parameter_degree,
                                   parameter_backend):
    u.interpolate(Constant(-1000.0))
    problem = NonlinearVariationalProblem(F, u, bcs, J)
    problem.set_bounds(lb, ub)

    solver = NonlinearVariationalSolver(problem)
    solver.parameters.update(snes_solver_parameters_bounds)
    u.interpolate(Constant(-1000.0))
    solver.solve()
    u.interpolate(Constant(-1000.0))
    solver.solve()
    assert u.vector().min() >= 0


@skip_if_not_PETSc
def test_snes_solver_bound_vectors(F, u, bcs, J,
                                   snes_solver_parameters_bounds,
                                   lb, ub, parameter_degree,
                                   parameter_backend):
    problem = NonlinearVariationalProblem(F, u, bcs, J)
    problem.set_bounds(lb, ub)

    solver = NonlinearVariationalSolver(problem)
    solver.parameters.update(snes_solver_parameters_bounds)
    u.interpolate(Constant(-1000.0))
    solver.solve()
    u.interpolate(Constant(-1000.0))
    solver.solve()
    assert u.vector().min() >= 0


@skip_if_not_PETSc
def test_snes_set_from_options():
    solver = PETScSNESSolver()
    PETScOptions.set("snes_atol", 1e-12)
    solver.set_from_options()



# Copyright (C) 2005-2010 Anders Logg
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# Recompile all ffc forms (use when FFC has been updated)
# This script should be run from the top level directory.

import os
import ffc

# Forms that need special options
use_error = ["AdaptivePoisson.ufl",
             "AdaptiveNavierStokes.ufl",
]

use_representation = {"HyperElasticity.ufl": "uflacs",
                      "CahnHilliard2D.ufl": "uflacs",
                      "CahnHilliard3D.ufl": "uflacs",
}

use_split = ["Poisson2D_1.ufl",
             "Poisson2D_2.ufl",
             "Poisson2D_3.ufl",
             "Poisson2D_4.ufl",
             "Poisson2D_5.ufl",
             "Poisson3D_1.ufl",
             "Poisson3D_2.ufl",
             "Poisson3D_3.ufl",
             "Poisson3D_4.ufl",
             "Poisson3D_5.ufl",
             "CahnHilliard2D.ufl",
             "CahnHilliard3D.ufl",
]

# Forms for which we don't want to generate functions for evaluating
# the basis
skip_basis = ["Poisson2D_5.ufl", "Poisson3D_4.ufl"]

# Directories to scan
subdirs = ["demo", "bench", "test"]

# Compile all form files
topdir = os.getcwd()
for subdir in subdirs:
    for root, dirs, files in os.walk(subdir):
        # Check for .ufl files
        formfiles = [f for f in files if f[-4:] == ".ufl"]
        if not formfiles:
            continue

        # Compile files
        os.chdir(root)
        print("Compiling %d forms in %s..." % (len(formfiles), root))
        for f in formfiles:
            args = ["-l", "dolfin"]
            args.append("-s")
            args.append("-O")

            args.append("-r")
            args.append(use_representation.get(f, "auto"))

            if f in use_split:
                args.append("-fsplit")

            if f in use_error:
                args.append("-e")

            if f in skip_basis:
                args.append("-fno-evaluate_basis")
                args.append("-fno-evaluate_basis_derivatives")

            args.append(f)

            command = "ffc " + " ".join(args) # + " >> compile.log"
            print(command)

            ret = ffc.main(args)
            if ret != 0:
                raise RuntimeError("Unable to compile form: %s/%s" % (root, f))
        os.chdir(topdir)



"Unit tests for XML input/output of MeshFunction (class XMLMeshFunction)"

# Copyright (C) 2011-2014 Garth N. Wells
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# Modified by Anders Logg 2011

import pytest
import os
from dolfin import *
from dolfin_utils.test import skip_in_parallel, fixture, cd_tempdir


@skip_in_parallel
def test_io_size_t(cd_tempdir):
    "Test input/output for size_t"

    # Write some data
    mesh = UnitSquareMesh(5, 5)
    f = MeshFunction("size_t", mesh, 1)
    f.set_all(0)
    f[2] = 3
    f[5] = 7

    filename = "XMLMeshFunction_test_io_size_t.xml"

    # Write
    output_file = File(filename)
    output_file << f

    # Read from file
    g = MeshFunction("size_t", mesh, 1)
    input_file = File(filename)
    input_file >> g

    # Check values
    for i in range(f.size()):
        assert f[i] == g[i]


@skip_in_parallel
def test_io_int(cd_tempdir):
    "Test input/output for int"

    # Write some data
    mesh = UnitSquareMesh(5, 5)
    f = MeshFunction("int", mesh, 1)
    f.set_all(0)
    f[2] = -3
    f[5] = 7

    filename = "XMLMeshFunction_test_io_int.xml"

    # Write
    output_file = File(filename)
    output_file << f

    # Read from file
    g = MeshFunction("int", mesh, 1)
    input_file = File(filename)
    input_file >> g

    # Check values
    for i in range(f.size()):
        assert f[i] == g[i]


@skip_in_parallel
def test_io_double(cd_tempdir):
    "Test input/output for double"

    # Write some data
    mesh = UnitSquareMesh(5, 5)
    f = MeshFunction("double", mesh, 1)
    f.set_all(0.0)
    f[2] = 3.14
    f[5] = 10000000.0

    filename = "XMLMeshFunction_test_io_double.xml"

    # Write
    output_file = File(filename)
    output_file << f

    # Read from file
    g = MeshFunction("double", mesh, 1)
    input_file = File(filename)
    input_file >> g

    # Check values
    for i in range(f.size()):
        assert f[i] == g[i]


@skip_in_parallel
def test_io_bool(cd_tempdir):
    "Test input/output for bool"

    # Write some data
    mesh = UnitSquareMesh(5, 5)
    f = MeshFunction("bool", mesh, 1)
    f.set_all(False)
    f[2] = True
    f[5] = False

    filename = "XMLMeshFunction_test_io_bool.xml"

    # Write
    output_file = File(filename)
    output_file << f

    # Read from file
    g = MeshFunction("bool", mesh, 1)
    input_file = File(filename)
    input_file >> g

    # Check values
    for i in range(f.size()):
        assert f[i] == g[i]



"""This demo program solves Poisson's equation

    - div grad u(x, y) = f(x, y)

on the unit square with source f given by

    f(x, y) =    -1.0 if (x - 0.33)^2 + (y - 0.67)^2 < 0.015
                  5.0 if 0.015 < (x - 0.33)^2 + (y - 0.67)^2 < 0.025
                 -1.0 if (x,y) in triangle( (0.55, 0.05), (0.95, 0.45), (0.55, 0.45) )
                  0.0 otherwise

and homogeneous Dirichlet boundary conditions.
"""

# Copyright (C) 2010 Kristian B. Oelgaard
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# Modified by Anders Logg 2011
#
# First added:  2010-07-23
# Last changed: 2012-11-12

from dolfin import *
import matplotlib.pyplot as plt


# Create mesh and define function space
mesh = UnitSquareMesh(64, 64)
V = FunctionSpace(mesh, "CG", 2)

# Define Dirichlet boundary (x = 0 or x = 1)
def boundary(x):
    return x[0] < DOLFIN_EPS or x[0] > 1.0 - DOLFIN_EPS or x[1] < DOLFIN_EPS or x[1] > 1.0 - DOLFIN_EPS

# Define boundary condition
u0 = Constant(0.0)
bc = DirichletBC(V, u0, boundary)

# Define variational problem
u = TrialFunction(V)
v = TestFunction(V)
x = SpatialCoordinate(mesh)

c0 = conditional(le( (x[0]-0.33)**2 + (x[1]-0.67)**2,  0.015), -1.0, 5.0)
c = conditional( le( (x[0]-0.33)**2 + (x[1]-0.67)**2,  0.025), c0, 0.0 )

t0 = conditional(ge( x[0],  0.55), -1.0, 0.0)
t1 = conditional(le( x[0],  0.95), t0, 0.0)
t2 = conditional(ge( x[1],  0.05), t1, 0.0)
t3 = conditional(le( x[1],  0.45), t2, 0.0)
t = conditional(ge( x[1] - x[0] - 0.05 + 0.55,  0.0), t3, 0.0)
f = c + t
a = inner(grad(u), grad(v))*dx
L = f*v*dx

# Compute solution
u = Function(V)
solve(a == L, u, bc)

# Save solution in VTK format
file = File("conditional.pvd")
file << u

# Plot solution
plot(u)
plt.show()



"""This script demonstrate how to use sub function assignment."""

# Copyright (C) 2013 Johan Hake
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# First added:  2013-11-08
# Last changed: 2013-11-08

from dolfin import *
import time
import matplotlib.pyplot as plt


# Create mesh and define function spaces
mesh = UnitSquareMesh(256, 256)
V = FunctionSpace(mesh, "CG", 1)
VV = VectorFunctionSpace(mesh, "CG", 1)

# Define function
v0 = Expression("sin(2*pi*x[0])*sin(2*pi*x[1])", degree=2)
v1 = Expression("cos(2*pi*x[0])*cos(2*pi*x[1])", degree=2)

vv1 = Function(VV)
vv2 = Function(VV)
vv3 = Function(VV)

u0 = Function(V)
u1 = Function(V)

u0.interpolate(v0)
u1.interpolate(v1)

# Compute projection (L2-projection)
t0 = time.time()
vv0 = project(as_vector((u0, u1)), V=VV)
tp = time.time() - t0

# Compute interpolation (evaluating dofs)
t0 = time.time()
vv1.interpolate(Expression(("v0", "v1"), v0=v0, v1=v1, degree=2))
ti = time.time() - t0

# Assign mixed function from two scalar functions
t0 = time.time()
assign(vv2, [u0, u1])
ta0 = time.time() - t0

# Assign mixed function from two scalar functions using FunctionAssigner
assigner = FunctionAssigner(VV, [V, V])
t0 = time.time()
assigner.assign(vv3, [u0, u1])
ta1 = time.time() - t0

# Plot functions
plt.figure()
plot(vv0, title="Projection; time=%.3fs" % tp)
plt.figure()
plot(vv1, title="Interpolation; time=%.3fs" % ti)
plt.figure()
plot(vv2, title="Assignment; time=%.3fs" % ta0)
plt.figure()
plot(vv3, title="Assignment (cached); time=%.3fs" % ta1)
plt.show()



# -*- coding: utf-8 -*-
"""This module provides a simple way to compute the interpolation of a
:py:class:`Function <dolfin.functions.function.Function>` or
:py:class:`Expression <dolfin.functions.expression.Expression>` onto a
finite element space.

"""

# Copyright (C) 2009-2011 Anders Logg
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.

from dolfin.function.function import Function
from dolfin.function.multimeshfunction import MultiMeshFunction
from dolfin.function.multimeshfunctionspace import MultiMeshFunctionSpace


def interpolate(v, V):
    """Return interpolation of a given function into a given finite
    element space.

    *Arguments*
        v
            a :py:class:`Function <dolfin.function.function.Function>` or
            a :py:class:`MultiMeshFunction
                         <dolfin.function.function.MultiMeshFunction>` or
            an :py:class:`Expression <dolfin.functions.expression.Expression>`
        V
            a :py:class:`FunctionSpace (standard, mixed, etc.)
            <dolfin.functions.functionspace.FunctionSpace>`
            or a :py:class:`MultiMeshFunctionSpace
            <dolfin.function.MultiMeshFunctionSpace>`.


    *Example of usage*

        .. code-block:: python

            v = Expression("sin(pi*x[0])")
            V = FunctionSpace(mesh, "Lagrange", 1)
            Iv = interpolate(v, V)

    """

    # Check arguments
    # if not isinstance(V, cpp.functionFunctionSpace):
    #     cpp.dolfin_error("interpolation.py",
    #                      "compute interpolation",
    #                      "Illegal function space for interpolation, not a FunctionSpace (%s)" % str(v))

    if isinstance(V, MultiMeshFunctionSpace):
        Pv = MultiMeshFunction(V)
    else:
        Pv = Function(V)

    # Compute interpolation
    if hasattr(v, "_cpp_object"):
        Pv.interpolate(v._cpp_object)
    else:
        Pv.interpolate(v)

    return Pv
